@@ -40,10 +40,10 @@ def test_reglob_tests(testfile):
 
 
 @pytest.fixture
-def home_env(xonsh_builtins):
+def home_env(xession):
     """Set `__xonsh__.env ` to a new Env instance on `xonsh_builtins`"""
-    xonsh_builtins.__xonsh__.env = Env(HOME=HOME_PATH)
-    return xonsh_builtins
+    xession.env = Env(HOME=HOME_PATH)
+    return xession
 
 
 @skip_if_on_windows
