@@ -5,8 +5,10 @@ import os
 import re
 import pathlib
 import datetime
-import itertools
+from random import shuffle
 from tempfile import TemporaryDirectory
+from threading import Thread
+from time import sleep
 
 import pytest
 
@@ -20,6 +22,7 @@ from xonsh.environ import (
     LsColors,
     default_value,
     Var,
+    InternalEnvironDict,
 )
 
 from tools import skip_if_on_unix
@@ -150,8 +153,47 @@ def test_swap_exception_replacement():
     assert env["VAR"] == "original value"
 
 
+def test_thread_local_swap():
+    env = Env(a="orig")
+    iter_count = 10
+    num_threads = 4
+    success_variables = [False] * (num_threads + 1)
+
+    def loop(index, swapped_values=None):
+        if swapped_values:
+            if env["a"] != "orig":
+                success_variables[index] = False
+                return
+            env.set_swapped_values(swapped_values)
+        for _ in range(iter_count):
+            if env["a"] != "swapped":
+                success_variables[index] = False
+                break
+            with env.swap(a=index):
+                sleep(0.1)
+                if env["a"] == index:
+                    success_variables[index] = True
+                else:
+                    success_variables[index] = False
+                    break
+            sleep(0.1)
+
+    with env.swap(a="swapped"):
+        threads = [
+            Thread(target=loop, args=(i, env.get_swapped_values()))
+            for i in range(1, num_threads + 1)
+        ]
+        for t in threads:
+            t.start()
+        loop(0)
+    for t in threads:
+        t.join()
+
+    assert all(success_variables)
+
+
 @skip_if_on_unix
-def test_locate_binary_on_windows(xonsh_builtins):
+def test_locate_binary_on_windows(xession):
     files = ("file1.exe", "FILE2.BAT", "file3.txt")
     with TemporaryDirectory() as tmpdir:
         tmpdir = os.path.realpath(tmpdir)
@@ -159,10 +201,8 @@ def test_locate_binary_on_windows(xonsh_builtins):
             fpath = os.path.join(tmpdir, fname)
             with open(fpath, "w") as f:
                 f.write(fpath)
-        xonsh_builtins.__xonsh__.env.update(
-            {"PATH": [tmpdir], "PATHEXT": [".COM", ".EXE", ".BAT"]}
-        )
-        xonsh_builtins.__xonsh__.commands_cache = CommandsCache()
+        xession.env.update({"PATH": [tmpdir], "PATHEXT": [".COM", ".EXE", ".BAT"]})
+        xession.commands_cache = CommandsCache()
         assert locate_binary("file1") == os.path.join(tmpdir, "file1.exe")
         assert locate_binary("file1.exe") == os.path.join(tmpdir, "file1.exe")
         assert locate_binary("file2") == os.path.join(tmpdir, "FILE2.BAT")
@@ -170,13 +210,13 @@ def test_locate_binary_on_windows(xonsh_builtins):
         assert locate_binary("file3") is None
 
 
-def test_event_on_envvar_change(xonsh_builtins):
+def test_event_on_envvar_change(xession):
     env = Env(TEST=0)
-    xonsh_builtins.__xonsh__.env = env
+    xession.env = env
     share = []
     # register
 
-    @xonsh_builtins.events.on_envvar_change
+    @xession.builtins.events.on_envvar_change
     def handler(name, oldvalue, newvalue, **kwargs):
         share.extend((name, oldvalue, newvalue))
 
@@ -186,13 +226,13 @@ def test_event_on_envvar_change(xonsh_builtins):
     assert share == ["TEST", 0, 1]
 
 
-def test_event_on_envvar_new(xonsh_builtins):
+def test_event_on_envvar_new(xession):
     env = Env()
-    xonsh_builtins.__xonsh__.env = env
+    xession.env = env
     share = []
     # register
 
-    @xonsh_builtins.events.on_envvar_new
+    @xession.builtins.events.on_envvar_new
     def handler(name, value, **kwargs):
         share.extend((name, value))
 
@@ -202,13 +242,13 @@ def test_event_on_envvar_new(xonsh_builtins):
     assert share == ["TEST", 1]
 
 
-def test_event_on_envvar_change_from_none_value(xonsh_builtins):
+def test_event_on_envvar_change_from_none_value(xession):
     env = Env(TEST=None)
-    xonsh_builtins.__xonsh__.env = env
+    xession.env = env
     share = []
     # register
 
-    @xonsh_builtins.events.on_envvar_change
+    @xession.builtins.events.on_envvar_change
     def handler(name, oldvalue, newvalue, **kwargs):
         share.extend((name, oldvalue, newvalue))
 
@@ -219,13 +259,13 @@ def test_event_on_envvar_change_from_none_value(xonsh_builtins):
 
 
 @pytest.mark.parametrize("val", [1, None, True, "ok"])
-def test_event_on_envvar_change_no_fire_when_value_is_same(val, xonsh_builtins):
+def test_event_on_envvar_change_no_fire_when_value_is_same(val, xession):
     env = Env(TEST=val)
-    xonsh_builtins.__xonsh__.env = env
+    xession.env = env
     share = []
     # register
 
-    @xonsh_builtins.events.on_envvar_change
+    @xession.builtins.events.on_envvar_change
     def handler(name, oldvalue, newvalue, **kwargs):
         share.extend((name, oldvalue, newvalue))
 
@@ -235,17 +275,17 @@ def test_event_on_envvar_change_no_fire_when_value_is_same(val, xonsh_builtins):
     assert share == []
 
 
-def test_events_on_envvar_called_in_right_order(xonsh_builtins):
+def test_events_on_envvar_called_in_right_order(xession):
     env = Env()
-    xonsh_builtins.__xonsh__.env = env
+    xession.env = env
     share = []
     # register
 
-    @xonsh_builtins.events.on_envvar_new
+    @xession.builtins.events.on_envvar_new
     def handler(name, value, **kwargs):
         share[:] = ["new"]
 
-    @xonsh_builtins.events.on_envvar_change
+    @xession.builtins.events.on_envvar_change
     def handler1(name, oldvalue, newvalue, **kwargs):
         share[:] = ["change"]
 
@@ -320,13 +360,13 @@ def test_lscolors_target(xonsh_builtins):
         ("pi", ("BACKGROUND_BLACK", "YELLOW"), None, "delete existing key"),
     ],
 )
-def test_lscolors_events(key_in, old_in, new_in, test, xonsh_builtins):
+def test_lscolors_events(key_in, old_in, new_in, test, xession):
     lsc = LsColors.fromstring("fi=0:di=01;34:pi=40;33")
     # corresponding colors: [('RESET',), ('BOLD_CYAN',), ('BOLD_CYAN',), ('BACKGROUND_BLACK', 'YELLOW')]
 
     event_fired = False
 
-    @xonsh_builtins.events.on_lscolors_change
+    @xession.builtins.events.on_lscolors_change
     def handler(key, oldvalue, newvalue, **kwargs):
         nonlocal old_in, new_in, key_in, event_fired
         assert (
@@ -334,7 +374,7 @@ def test_lscolors_events(key_in, old_in, new_in, test, xonsh_builtins):
         ), "Old and new event values match"
         event_fired = True
 
-    xonsh_builtins.__xonsh__.env["LS_COLORS"] = lsc
+    xession.env["LS_COLORS"] = lsc
 
     if new_in is None:
         lsc.pop(key_in, "argle")
@@ -439,7 +479,7 @@ def test_register_var_path():
     # Empty string is None to avoid uncontrolled converting empty string to Path('.')
     path = ""
     env["MY_PATH_VAR"] = path
-    assert env["MY_PATH_VAR"] == None
+    assert env["MY_PATH_VAR"] is None
 
     with pytest.raises(TypeError):
         env["MY_PATH_VAR"] = 42
@@ -549,3 +589,49 @@ def test_env_get_defaults():
 def test_var_with_default_initer(val, validator):
     var = Var.with_default(val)
     assert var.validate.__name__ == validator
+
+
+def test_thread_local_dict():
+    d = InternalEnvironDict()
+    d["a"] = 1
+    assert d["a"] == 1
+    d.set_locally("a", 2)
+    assert d["a"] == 2
+    d.set_locally("a", 3)
+    assert d["a"] == 3
+    d["a"] = 4
+    assert d["a"] == 4
+    d.del_locally("a")
+    assert d["a"] == 1
+    d.set_locally("a", 5)
+    assert d.pop("a") == 5
+    assert d["a"] == 1
+    d.set_locally("a", 6)
+    assert d.popitem() == ("a", 6)
+    assert d["a"] == 1
+    assert d.pop("a", "nope") == 1
+    assert d.pop("a", "nope") == "nope"
+    assert "a" not in d
+
+
+def test_thread_local_dict_multiple():
+    d = InternalEnvironDict()
+    num_threads = 5
+    thread_values = [None] * num_threads
+
+    def thread(i):
+        d.set_locally("a", i ** 2)
+        sleep(0.1)
+        thread_values[i] = d["a"]
+        sleep(0.1)
+        d.del_locally("a")
+        sleep(0.1)
+
+    threads = [Thread(target=thread, args=(i,)) for i in range(num_threads)]
+    shuffle(threads)
+    for t in threads:
+        t.start()
+    for t in threads:
+        t.join()
+
+    assert thread_values == [i ** 2 for i in range(num_threads)]
