@@ -1,13 +1,13 @@
 # -*- coding: utf-8 -*-
 """Tests the xonsh parser."""
 import ast
-import builtins
 import textwrap
 import itertools
 
 import pytest
 
 from xonsh.ast import AST, With, Pass, Str, Call
+from xonsh.built_ins import XSH
 from xonsh.parser import Parser
 from xonsh.parsers.fstring_adaptor import FStringAdaptor
 
@@ -19,7 +19,7 @@ def xonsh_builtins_autouse(xonsh_builtins):
     return xonsh_builtins
 
 
-PARSER = Parser(lexer_optimize=False, yacc_optimize=False, yacc_debug=True)
+PARSER = Parser(yacc_optimize=False, yacc_debug=True)
 
 
 def check_ast(inp, run=True, mode="eval", debug_level=0):
@@ -43,8 +43,7 @@ def check_stmts(inp, run=True, mode="exec", debug_level=0):
 
 
 def check_xonsh_ast(xenv, inp, run=True, mode="eval", debug_level=0, return_obs=False):
-    __tracebackhide__ = True
-    builtins.__xonsh__.env = xenv
+    XSH.env = xenv
     obs = PARSER.parse(inp, debug_level=debug_level)
     if obs is None:
         return  # comment only
@@ -61,6 +60,12 @@ def check_xonsh(xenv, inp, run=True, mode="exec"):
     check_xonsh_ast(xenv, inp, run=run, mode=mode)
 
 
+def eval_code(inp, mode="eval", **loc_vars):
+    obs = PARSER.parse(inp, debug_level=1)
+    bytecode = compile(obs, "<test-xonsh-ast>", mode)
+    return eval(bytecode, loc_vars)
+
+
 #
 # Tests
 #
@@ -162,7 +167,7 @@ def test_fstring_adaptor(inp, exp):
     assert isinstance(joined_str_node, ast.JoinedStr)
     node = ast.Expression(body=joined_str_node)
     code = compile(node, "<test_fstring_adaptor>", mode="eval")
-    builtins.__xonsh__.env = {"HOME": "/foo/bar", "FOO": "HO", "BAR": "ME"}
+    XSH.env = {"HOME": "/foo/bar", "FOO": "HO", "BAR": "ME"}
     obs = eval(code)
     assert exp == obs
 
@@ -390,6 +395,39 @@ def test_if_else_expr_expr():
     check_ast("42+5 if 1 == 2 else 65-5")
 
 
+def test_subscription_syntaxes():
+    assert eval_code("[1, 2, 3][-1]") == 3
+    assert eval_code("[1, 2, 3][-1]") == 3
+    assert eval_code("'string'[-1]") == "g"
+
+
+@pytest.fixture
+def arr_container():
+    # like numpy.r_
+    class Arr:
+        def __getitem__(self, item):
+            return item
+
+    return Arr()
+
+
+def test_subscription_special_syntaxes(arr_container):
+    assert eval_code("arr[1, 2, 3]", arr=arr_container) == (1, 2, 3)
+    # dataframe
+    assert eval_code('arr[["a", "b"]]', arr=arr_container) == ["a", "b"]
+
+
+# todo: enable this test
+@pytest.mark.xfail
+def test_subscription_special_syntaxes_2(arr_container):
+    # aliases
+    d = {}
+    eval_code("d[arr.__name__]=True", arr=arr_container, d=d)
+    assert d == {"Arr": True}
+    # extslice
+    assert eval_code('arr[:, "2"]') == 2
+
+
 def test_str_idx():
     check_ast('"hello"[0]')
 
@@ -1373,6 +1411,10 @@ def test_equals_annotation():
     check_stmts("x : int = 42")
 
 
+def test_equals_annotation_empty():
+    check_stmts("x : int")
+
+
 def test_dict_keys():
     check_stmts('x = {"x": 1}\nx.keys()')
 
@@ -1566,7 +1608,7 @@ def test_yield_x_y():
 
 
 @skip_if_pre_3_8
-def test_return_x_starexpr():
+def test_yield_x_starexpr():
     check_stmts("yield x, *[y, z]", False)
 
 
@@ -2138,8 +2180,16 @@ def test_dollar_sub():
     check_xonsh_ast({}, "$(ls)", False)
 
 
-def test_dollar_sub_space():
-    check_xonsh_ast({}, "$(ls )", False)
+@pytest.mark.parametrize(
+    "expr",
+    [
+        "$(ls )",
+        "$( ls)",
+        "$( ls )",
+    ],
+)
+def test_dollar_sub_space(expr):
+    check_xonsh_ast({}, expr, False)
 
 
 def test_ls_dot():
@@ -2204,8 +2254,16 @@ def test_bang_sub():
     check_xonsh_ast({}, "!(ls)", False)
 
 
-def test_bang_sub_space():
-    check_xonsh_ast({}, "!(ls )", False)
+@pytest.mark.parametrize(
+    "expr",
+    [
+        "!(ls )",
+        "!( ls)",
+        "!( ls )",
+    ],
+)
+def test_bang_sub_space(expr):
+    check_xonsh_ast({}, expr, False)
 
 
 def test_bang_ls_dot():
