@@ -5,14 +5,15 @@ import re
 import sys
 import inspect
 import argparse
-import builtins
 import collections.abc as cabc
+import typing as tp
 
+from xonsh.built_ins import XSH
 from xonsh.lazyasd import lazyobject
 from xonsh.dirstack import cd, pushd, popd, dirs, _get_cwd
 from xonsh.environ import locate_binary, make_args_env
 from xonsh.foreign_shells import foreign_shell_data
-from xonsh.jobs import jobs, fg, bg, clean_jobs
+from xonsh.jobs import jobs, fg, bg, clean_jobs, disown
 from xonsh.platform import (
     ON_ANACONDA,
     ON_DARWIN,
@@ -22,6 +23,7 @@ from xonsh.platform import (
     ON_OPENBSD,
     ON_DRAGONFLY,
     ON_POSIX,
+    IN_APPIMAGE,
 )
 from xonsh.tools import (
     XonshError,
@@ -33,10 +35,12 @@ from xonsh.tools import (
     ALIAS_KWARG_NAMES,
     unthreadable,
     print_color,
+    to_repr_pretty_,
 )
 from xonsh.timings import timeit_alias
 from xonsh.xontribs import xontribs_main
 from xonsh.ast import isexpression
+from xonsh.cli_utils import ArgParserAlias, Arg, Annotated
 
 import xonsh.completers._aliases as xca
 import xonsh.history.main as xhm
@@ -92,7 +96,7 @@ class Aliases(cabc.MutableMapping):
         if callable(value):
             return partial_eval_alias(value, acc_args=acc_args)
         else:
-            expand_path = builtins.__xonsh__.expand_path
+            expand_path = XSH.expand_path
             token, *rest = map(expand_path, value)
             if token in seen_tokens or token not in self._raw:
                 # ^ Making sure things like `egrep=egrep --color=auto` works,
@@ -107,15 +111,19 @@ class Aliases(cabc.MutableMapping):
                 acc_args = rest + list(acc_args)
                 return self.eval_alias(self._raw[token], seen_tokens, acc_args)
 
-    def expand_alias(self, line):
+    def expand_alias(self, line: str, cursor_index: int) -> str:
         """Expands any aliases present in line if alias does not point to a
         builtin function and if alias is only a single command.
+        The command won't be expanded if the cursor's inside/behind it.
         """
-        word = line.split(" ", 1)[0]
-        if word in builtins.aliases and isinstance(self.get(word), cabc.Sequence):
+        word = (line.split(maxsplit=1) or [""])[0]
+        if word in XSH.aliases and isinstance(self.get(word), cabc.Sequence):  # type: ignore
             word_idx = line.find(word)
-            expansion = " ".join(self.get(word))
-            line = line[:word_idx] + expansion + line[word_idx + len(word) :]
+            word_edge = word_idx + len(word)
+            if cursor_index > word_edge:
+                # the cursor isn't inside/behind the word
+                expansion = " ".join(self.get(word))
+                line = line[:word_idx] + expansion + line[word_edge:]
         return line
 
     #
@@ -133,7 +141,7 @@ class Aliases(cabc.MutableMapping):
                 self._raw[key] = ExecAlias(val, filename=f)
             elif isexpression(val):
                 # expansion substitution
-                lexer = builtins.__xonsh__.execer.parser.lexer
+                lexer = XSH.execer.parser.lexer
                 self._raw[key] = list(map(strip_simple_quotes, lexer.split(val)))
             else:
                 # need to exec alias
@@ -179,14 +187,7 @@ class Aliases(cabc.MutableMapping):
             self.__class__.__module__, self.__class__.__name__, self._raw
         )
 
-    def _repr_pretty_(self, p, cycle):
-        name = "{0}.{1}".format(self.__class__.__module__, self.__class__.__name__)
-        with p.group(0, name + "(", ")"):
-            if cycle:
-                p.text("...")
-            elif len(self):
-                p.break_()
-                p.pretty(dict(self))
+    _repr_pretty_ = to_repr_pretty_
 
 
 class ExecAlias:
@@ -199,17 +200,26 @@ class ExecAlias:
         src : str
             Source code that will be
         """
-        self.src = src if src.endswith("\n") else src + "\n"
+        self.src = src
         self.filename = filename
 
     def __call__(
         self, args, stdin=None, stdout=None, stderr=None, spec=None, stack=None
     ):
-        execer = builtins.__xonsh__.execer
+        execer = XSH.execer
         frame = stack[0][0]  # execute as though we are at the call site
-        execer.exec(
-            self.src, glbs=frame.f_globals, locs=frame.f_locals, filename=self.filename
-        )
+
+        alias_args = {"args": args}
+        for i, a in enumerate(args):
+            alias_args[f"arg{i}"] = a
+
+        with XSH.env.swap(alias_args):
+            execer.exec(
+                self.src,
+                glbs=frame.f_globals,
+                locs=frame.f_locals,
+                filename=self.filename,
+            )
 
     def __repr__(self):
         return "ExecAlias({0!r}, filename={1!r})".format(self.src, self.filename)
@@ -349,14 +359,14 @@ def xonsh_exit(args, stdin=None):
     if not clean_jobs():
         # Do not exit if jobs not cleaned up
         return None, None
-    builtins.__xonsh__.exit = True
+    XSH.exit = True
     print()  # gimme a newline
     return None, None
 
 
 def xonsh_reset(args, stdin=None):
-    """ Clears __xonsh__.ctx"""
-    builtins.__xonsh__.ctx.clear()
+    """Clears __xonsh__.ctx"""
+    XSH.ctx.clear()
 
 
 @lazyobject
@@ -488,7 +498,7 @@ def _SOURCE_FOREIGN_PARSER():
 
 def source_foreign(args, stdin=None, stdout=None, stderr=None):
     """Sources a file written in a foreign shell language."""
-    env = builtins.__xonsh__.env
+    env = XSH.env
     ns = _SOURCE_FOREIGN_PARSER.parse_args(args)
     ns.suppress_skip_message = (
         env.get("FOREIGN_ALIASES_SUPPRESS_SKIP_MESSAGE")
@@ -539,7 +549,7 @@ def source_foreign(args, stdin=None, stdout=None, stderr=None):
         if k not in fsenv:
             env.pop(k, None)
     # Update aliases
-    baliases = builtins.aliases
+    baliases = XSH.aliases
     for k, v in fsaliases.items():
         if k in baliases and v == baliases[k]:
             continue  # no change from original
@@ -564,7 +574,7 @@ def source_alias(args, stdin=None):
     If sourced file isn't found in cwd, search for file along $PATH to source
     instead.
     """
-    env = builtins.__xonsh__.env
+    env = XSH.env
     encoding = env.get("XONSH_ENCODING")
     errors = env.get("XONSH_ENCODING_ERRORS")
     for i, fname in enumerate(args):
@@ -591,11 +601,11 @@ def source_alias(args, stdin=None):
             src = fp.read()
         if not src.endswith("\n"):
             src += "\n"
-        ctx = builtins.__xonsh__.ctx
+        ctx = XSH.ctx
         updates = {"__file__": fpath, "__name__": os.path.abspath(fpath)}
         with env.swap(**make_args_env(args[i + 1 :])), swap_values(ctx, updates):
             try:
-                builtins.execx(src, "exec", ctx, filename=fpath)
+                XSH.builtins.execx(src, "exec", ctx, filename=fpath)
             except Exception:
                 print_color(
                     "{RED}You may be attempting to source non-xonsh file! "
@@ -625,28 +635,38 @@ def source_cmd(args, stdin=None):
     args.append("--envcmd=set")
     args.append("--seterrpostcmd=if errorlevel 1 exit 1")
     args.append("--use-tmpfile=1")
-    with builtins.__xonsh__.env.swap(PROMPT="$P$G"):
+    with XSH.env.swap(PROMPT="$P$G"):
         return source_foreign(args, stdin=stdin)
 
 
-def xexec(args, stdin=None):
-    """exec [-h|--help] [-cl] [-a name] command [args...]
+def xexec_fn(
+    command: Annotated[tp.List[str], Arg(nargs=argparse.REMAINDER)],
+    login=False,
+    clean=False,
+    name="",
+    _stdin=None,
+):
+    """exec (also aliased as xexec) uses the os.execvpe() function to
+    replace the xonsh process with the specified program.
 
-    exec (also aliased as xexec) uses the os.execvpe() function to
-    replace the xonsh process with the specified program. This provides
-    the functionality of the bash 'exec' builtin::
+    This provides the functionality of the bash 'exec' builtin::
 
         >>> exec bash -l -i
         bash $
 
-    The '-h' and '--help' options print this message and exit.
-    If the '-l' option is supplied, the shell places a dash at the
-    beginning of the zeroth argument passed to command to simulate login
-    shell.
-    The '-c' option causes command to be executed with an empty environment.
-    If '-a' is supplied, the shell passes name as the zeroth argument
-    to the executed command.
-
+    Parameters
+    ----------
+    command
+        program to launch along its arguments
+    login : -l, --login
+        the shell places a dash at the
+        beginning of the zeroth argument passed to command to simulate login
+        shell.
+    clean : -c, --clean
+        causes command to be executed with an empty environment.
+    name : -a, --name
+        the shell passes name as the zeroth argument
+        to the executed command.
 
     Notes
     -----
@@ -658,66 +678,38 @@ def xexec(args, stdin=None):
     explicitly with ![exec command]. For more details, please see
     http://xon.sh/faq.html#exec.
     """
-    if len(args) == 0:
-        return (None, "xonsh: exec: no args specified\n", 1)
-
-    parser = argparse.ArgumentParser(add_help=False)
-    parser.add_argument("-h", "--help", action="store_true")
-    parser.add_argument("-l", dest="login", action="store_true")
-    parser.add_argument("-c", dest="clean", action="store_true")
-    parser.add_argument("-a", dest="name", nargs="?")
-    parser.add_argument("command", nargs=argparse.REMAINDER)
-    args = parser.parse_args(args)
-
-    if args.help:
-        return inspect.getdoc(xexec)
-
-    if len(args.command) == 0:
+    if len(command) == 0:
         return (None, "xonsh: exec: no command specified\n", 1)
 
-    command = args.command[0]
-    if args.name is not None:
-        args.command[0] = args.name
-    if args.login:
-        args.command[0] = "-{}".format(args.command[0])
+    cmd = command[0]
+    if name:
+        command[0] = name
+    if login:
+        command[0] = "-{}".format(command[0])
 
     denv = {}
-    if not args.clean:
-        denv = builtins.__xonsh__.env.detype()
+    if not clean:
+        denv = XSH.env.detype()
 
     try:
-        os.execvpe(command, args.command, denv)
+        os.execvpe(cmd, command, denv)
     except FileNotFoundError as e:
         return (
             None,
-            "xonsh: exec: file not found: {}: {}"
-            "\n".format(e.args[1], args.command[0]),
+            "xonsh: exec: file not found: {}: {}" "\n".format(e.args[1], command[0]),
             1,
         )
 
 
-class AWitchAWitch(argparse.Action):
-    SUPPRESS = "==SUPPRESS=="
-
-    def __init__(
-        self, option_strings, version=None, dest=SUPPRESS, default=SUPPRESS, **kwargs
-    ):
-        super().__init__(
-            option_strings=option_strings, dest=dest, default=default, nargs=0, **kwargs
-        )
-
-    def __call__(self, parser, namespace, values, option_string=None):
-        import webbrowser
-
-        webbrowser.open("https://github.com/xonsh/xonsh/commit/f49b400")
-        parser.exit()
+xexec = ArgParserAlias(func=xexec_fn, has_args=True, prog="xexec")
 
 
-def xonfig(args, stdin=None):
+@lazyobject
+def xonfig():
     """Runs the xonsh configuration utility."""
     from xonsh.xonfig import xonfig_main  # lazy import
 
-    return xonfig_main(args)
+    return xonfig_main
 
 
 @unthreadable
@@ -741,9 +733,9 @@ def showcmd(args, stdin=None):
     optional arguments:
       -h, --help            show this help message and exit
 
-    Example:
-    -------
-      >>> showcmd echo $USER can't hear "the sea"
+    Examples
+    --------
+      >>> showcmd echo $USER "can't" hear "the sea"
       ['echo', 'I', "can't", 'hear', 'the sea']
     """
     if len(args) == 0 or (len(args) == 1 and args[0] in {"-h", "--help"}):
@@ -765,7 +757,7 @@ def detect_xpip_alias():
 
     basecmd = [sys.executable, "-m", "pip"]
     try:
-        if ON_WINDOWS:
+        if ON_WINDOWS or IN_APPIMAGE:
             # XXX: Does windows have an installation mode that requires UAC?
             return basecmd
         elif not os.access(os.path.dirname(sys.executable), os.W_OK):
@@ -787,6 +779,7 @@ def make_default_aliases():
         "jobs": jobs,
         "fg": fg,
         "bg": bg,
+        "disown": disown,
         "EOF": xonsh_exit,
         "exit": xonsh_exit,
         "quit": xonsh_exit,
