@@ -7,11 +7,16 @@ and returns whether or not the process can be run in the background (returns
 True) or must be run the foreground (returns False).
 """
 import os
+import pickle
+import sys
+import threading
 import time
-import builtins
 import argparse
 import collections.abc as cabc
+import typing as tp
+from pathlib import Path
 
+from xonsh.built_ins import XSH
 from xonsh.platform import ON_WINDOWS, ON_POSIX, pathbasename
 from xonsh.tools import executables_in
 from xonsh.lazyasd import lazyobject
@@ -25,19 +30,32 @@ class CommandsCache(cabc.Mapping):
     the command has an alias.
     """
 
+    CACHE_FILE = "commands-cache.pickle"
+
     def __init__(self):
         self._cmds_cache = {}
         self._path_checksum = None
         self._alias_checksum = None
         self._path_mtime = -1
         self.threadable_predictors = default_threadable_predictors()
+        self._loaded_pickled = False
+
+        # Path to the cache-file where all commands/aliases are cached for pre-loading"""
+        env = XSH.env
+        self.cache_file = (
+            (Path(env["XONSH_DATA_DIR"]).joinpath(self.CACHE_FILE).resolve())
+            if env is not None
+            and "XONSH_DATA_DIR" in env
+            and env.get("COMMANDS_CACHE_SAVE_INTERMEDIATE")
+            else None
+        )
 
     def __contains__(self, key):
         _ = self.all_commands
         return self.lazyin(key)
 
     def __iter__(self):
-        for cmd, (path, is_alias) in self.all_commands.items():
+        for cmd, (path, _) in self.all_commands.items():
             if ON_WINDOWS and path is not None:
                 # All command keys are stored in uppercase on Windows.
                 # This ensures the original command name is returned.
@@ -58,58 +76,120 @@ class CommandsCache(cabc.Mapping):
     @staticmethod
     def get_possible_names(name):
         """Generates the possible `PATHEXT` extension variants of a given executable
-         name on Windows as a list, conserving the ordering in `PATHEXT`.
-         Returns a list as `name` being the only item in it on other platforms."""
+        name on Windows as a list, conserving the ordering in `PATHEXT`.
+        Returns a list as `name` being the only item in it on other platforms."""
         if ON_WINDOWS:
-            pathext = builtins.__xonsh__.env.get("PATHEXT", [])
+            pathext = XSH.env.get("PATHEXT", [])
             name = name.upper()
             return [name + ext for ext in ([""] + pathext)]
         else:
             return [name]
 
     @staticmethod
-    def remove_dups(p):
-        ret = list()
-        for e in p:
-            if e not in ret:
-                ret.append(e)
-        return ret
+    def remove_dups(paths):
+        cont = set()
+        for p in map(os.path.realpath, paths):
+            if p not in cont:
+                cont.add(p)
+                if os.path.isdir(p):
+                    yield p
+
+    def _update_if_changed(self, paths: tp.Tuple[str, ...], aliases):
+        # did PATH change?
+        path_hash = hash(paths)
+        yield path_hash == self._path_checksum
+        self._path_checksum = path_hash
+
+        # did aliases change?
+        al_hash = hash(frozenset(aliases))
+        yield al_hash == self._alias_checksum
+        self._alias_checksum = al_hash
+
+        # did the contents of any directory in PATH change?
+        max_mtime = max(map(lambda path: os.stat(path).st_mtime, paths), default=0)
+        yield max_mtime <= self._path_mtime
+        self._path_mtime = max_mtime
 
     @property
     def all_commands(self):
-        paths = builtins.__xonsh__.env.get("PATH", [])
-        paths = CommandsCache.remove_dups(paths)
-        path_immut = tuple(x for x in paths if os.path.isdir(x))
-        # did PATH change?
-        path_hash = hash(path_immut)
-        cache_valid = path_hash == self._path_checksum
-        self._path_checksum = path_hash
-        # did aliases change?
-        alss = getattr(builtins, "aliases", dict())
-        al_hash = hash(frozenset(alss))
-        cache_valid = cache_valid and al_hash == self._alias_checksum
-        self._alias_checksum = al_hash
-        # did the contents of any directory in PATH change?
-        max_mtime = 0
-        for path in path_immut:
-            mtime = os.stat(path).st_mtime
-            if mtime > max_mtime:
-                max_mtime = mtime
-        cache_valid = cache_valid and (max_mtime <= self._path_mtime)
-        self._path_mtime = max_mtime
-        if cache_valid:
+        env = XSH.env
+        path = [] if env is None else XSH.env.get("PATH", [])
+        path_immut = tuple(CommandsCache.remove_dups(path))
+        alss = getattr(XSH, "aliases", dict())
+        (
+            has_path_changed,
+            has_alias_changed,
+            has_any_path_updated,
+        ) = tuple(self._update_if_changed(path_immut, alss))
+        if has_path_changed and has_any_path_updated:
+            if not has_alias_changed:
+                for cmd, alias in alss.items():
+                    key = cmd.upper() if ON_WINDOWS else cmd
+                    if key in self._cmds_cache:
+                        self._cmds_cache[key] = (self._cmds_cache[key][0], alias)
+                    else:
+                        self._cmds_cache[key] = (cmd, True)
+                # save the changes to cache as well
+                self.set_cmds_cache(self._cmds_cache)
             return self._cmds_cache
+
+        if self.cache_file and self.cache_file.exists():
+            # pickle the result only if XONSH_DATA_DIR is set
+            if not self._loaded_pickled:
+                # first time load the commands from cache-file
+                self._cmds_cache = self.get_cached_commands()
+                self._loaded_pickled = True
+            # also start a thread that updates the cache in the bg
+            worker = threading.Thread(
+                target=self._update_cmds_cache,
+                args=[path_immut, alss],
+                daemon=True,
+            )
+            worker.start()
+        else:
+            self._update_cmds_cache(path_immut, alss)
+        return self._cmds_cache
+
+    def _update_cmds_cache(
+        self, paths: tp.Sequence[str], aliases: tp.Dict[str, str]
+    ) -> tp.Dict[str, tp.Any]:
+        """Update the cmds_cache variable in background without slowing down parseLexer"""
+        env = XSH.env or {}  # type: ignore
+
         allcmds = {}
-        for path in reversed(path_immut):
+        for path in reversed(paths):
             # iterate backwards so that entries at the front of PATH overwrite
             # entries at the back.
             for cmd in executables_in(path):
                 key = cmd.upper() if ON_WINDOWS else cmd
-                allcmds[key] = (os.path.join(path, cmd), alss.get(key, None))
-        for cmd in alss:
+                allcmds[key] = (os.path.join(path, cmd), aliases.get(key, None))
+
+        warn_cnt = env.get("COMMANDS_CACHE_SIZE_WARNING")
+        if warn_cnt and len(allcmds) > warn_cnt:
+            print(
+                f"Warning! Found {len(allcmds):,} executable files in the PATH directories!",
+                file=sys.stderr,
+            )
+
+        for cmd in aliases:
             if cmd not in allcmds:
                 key = cmd.upper() if ON_WINDOWS else cmd
-                allcmds[key] = (cmd, True)
+                allcmds[key] = (cmd, True)  # type: ignore
+        return self.set_cmds_cache(allcmds)
+
+    def get_cached_commands(self) -> tp.Dict[str, str]:
+        if self.cache_file and self.cache_file.exists():
+            try:
+                return pickle.loads(self.cache_file.read_bytes()) or {}
+            except Exception:
+                # the file is corrupt
+                self.cache_file.unlink(missing_ok=True)
+        return {}
+
+    def set_cmds_cache(self, allcmds: tp.Dict[str, tp.Any]) -> tp.Dict[str, tp.Any]:
+        """write cmds to cache-file and instance-attribute"""
+        if self.cache_file:
+            self.cache_file.write_bytes(pickle.dumps(allcmds))
         self._cmds_cache = allcmds
         return allcmds
 
@@ -154,10 +234,10 @@ class CommandsCache(cabc.Mapping):
         Parameters
         ----------
         name : str
-                name of binary to search for
+            name of binary to search for
         ignore_alias : bool, optional
-                Force return of binary path even if alias of ``name`` exists
-                (default ``False``)
+            Force return of binary path even if alias of ``name`` exists
+            (default ``False``)
         """
         # make sure the cache is up to date by accessing the property
         _ = self.all_commands
@@ -169,10 +249,10 @@ class CommandsCache(cabc.Mapping):
         Parameters
         ----------
         name : str
-                name of binary to search for
+            name of binary to search for
         ignore_alias : bool, optional
-                Force return of binary path even if alias of ``name`` exists
-                (default ``False``)
+            Force return of binary path even if alias of ``name`` exists
+            (default ``False``)
         """
         possibilities = self.get_possible_names(name)
         if ON_WINDOWS:
@@ -254,7 +334,7 @@ class CommandsCache(cabc.Mapping):
         """
         # alias stuff
         if not os.path.isabs(cmd0) and os.sep not in cmd0:
-            alss = getattr(builtins, "aliases", dict())
+            alss = getattr(XSH, "aliases", dict())
             if cmd0 in alss:
                 return self.default_predictor_alias(cmd0)
 
@@ -268,10 +348,10 @@ class CommandsCache(cabc.Mapping):
 
     def default_predictor_alias(self, cmd0):
         alias_recursion_limit = (
-            10
-        )  # this limit is se to handle infinite loops in aliases definition
+            10  # this limit is se to handle infinite loops in aliases definition
+        )
         first_args = []  # contains in reverse order args passed to the aliased command
-        alss = getattr(builtins, "aliases", dict())
+        alss = getattr(XSH, "aliases", dict())
         while cmd0 in alss:
             alias_name = alss[cmd0]
             if isinstance(alias_name, (str, bytes)) or not isinstance(
@@ -431,7 +511,7 @@ def predict_env(args):
         if args[i] and args[i][0] != "-" and "=" not in args[i]:
             # args[i] is the command and the following is its arguments
             # so args[i:] is used to predict if the command is threadable
-            return builtins.__xonsh__.commands_cache.predict_threadable(args[i:])
+            return XSH.commands_cache.predict_threadable(args[i:])
     return True
 
 
@@ -441,60 +521,82 @@ def default_threadable_predictors():
     """
     # alphabetical, for what it is worth.
     predictors = {
-        "aurman": predict_false,
         "asciinema": predict_help_ver,
+        "aurman": predict_false,
+        "awk": predict_true,
         "bash": predict_shell,
-        "csh": predict_shell,
+        "cat": predict_false,
         "clear": predict_false,
         "cls": predict_false,
         "cmd": predict_shell,
         "cryptop": predict_false,
+        "cryptsetup": predict_true,
+        "csh": predict_shell,
         "curl": predict_true,
+        "elvish": predict_shell,
+        "emacsclient": predict_false,
         "env": predict_env,
         "ex": predict_false,
-        "emacsclient": predict_false,
         "fish": predict_shell,
+        "gawk": predict_true,
+        "ghci": predict_help_ver,
+        "git": predict_true,
         "gvim": predict_help_ver,
         "hg": predict_hg,
         "htop": predict_help_ver,
         "ipython": predict_shell,
+        "julia": predict_shell,
         "ksh": predict_shell,
         "less": predict_help_ver,
         "ls": predict_true,
         "man": predict_help_ver,
+        "mc": predict_false,
         "more": predict_help_ver,
-        "mvim": predict_help_ver,
         "mutt": predict_help_ver,
+        "mvim": predict_help_ver,
         "nano": predict_help_ver,
+        "nmcli": predict_true,
         "nvim": predict_false,
+        "percol": predict_false,
         "ponysay": predict_help_ver,
         "psql": predict_false,
+        "push": predict_shell,
+        "pv": predict_false,
         "python": predict_shell,
         "python2": predict_shell,
         "python3": predict_shell,
-        "repo": predict_help_ver,
         "ranger": predict_help_ver,
+        "repo": predict_help_ver,
         "rview": predict_false,
         "rvim": predict_false,
+        "rwt": predict_shell,
         "scp": predict_false,
         "sh": predict_shell,
         "ssh": predict_false,
         "startx": predict_false,
         "sudo": predict_help_ver,
         "sudoedit": predict_help_ver,
+        "systemctl": predict_true,
         "tcsh": predict_shell,
         "telnet": predict_false,
-        "tput": predict_false,
         "top": predict_help_ver,
+        "tput": predict_false,
+        "udisksctl": predict_true,
+        "unzip": predict_true,
         "vi": predict_false,
         "view": predict_false,
         "vim": predict_false,
         "vimpager": predict_help_ver,
         "weechat": predict_help_ver,
+        "wget": predict_true,
         "xclip": predict_help_ver,
+        "xdg-open": predict_false,
         "xo": predict_help_ver,
-        "xonsh": predict_shell,
         "xon.sh": predict_shell,
+        "xonsh": predict_shell,
+        "yes": predict_false,
+        "zip": predict_true,
+        "zipinfo": predict_true,
         "zsh": predict_shell,
     }
     return predictors
