@@ -15,7 +15,7 @@ import functools
 import subprocess
 import typing as tp
 
-__version__ = "0.2.6"
+__version__ = "0.2.7"
 
 
 @functools.lru_cache(1)
@@ -288,6 +288,10 @@ def bash_completions(
     paths=None,
     command=None,
     quote_paths=_bash_quote_paths,
+    line_args=None,
+    opening_quote="",
+    closing_quote="",
+    arg_index=None,
     **kwargs
 ):
     """Completes based on results from BASH completion.
@@ -320,6 +324,15 @@ def bash_completions(
         this as the default is acceptable 99+% of the time. This function should
         return a set of the new paths and a boolean for whether the paths were
         quoted.
+    line_args : list of str, optional
+        A list of the args in the current line to be used instead of ``line.split()``.
+        This is usefull with a space in an argument, e.g. ``ls 'a dir/'<TAB>``.
+    opening_quote : str, optional
+        The current argument's opening quote. This is passed to the `quote_paths` function.
+    closing_quote : str, optional
+        The closing quote that **should** be used. This is also passed to the `quote_paths` function.
+    arg_index : int, optional
+        The current prefix's index in the args.
 
     Returns
     -------
@@ -333,26 +346,33 @@ def bash_completions(
     if prefix.startswith("$"):  # do not complete env variables
         return set(), 0
 
-    splt = line.split()
+    splt = line_args or line.split()
     cmd = splt[0]
-    idx = n = 0
+    cmd = os.path.basename(cmd)
     prev = ""
-    for n, tok in enumerate(splt):
-        if tok == prefix:
-            idx = line.find(prefix, idx)
-            if idx >= begidx:
-                break
-        prev = tok
 
-    if len(prefix) == 0:
-        prefix_quoted = '""'
-        n += 1
+    if arg_index is not None:
+        n = arg_index
+        if arg_index > 0:
+            prev = splt[arg_index - 1]
     else:
-        prefix_quoted = shlex.quote(prefix)
+        # find `n` and `prev` by ourselves
+        idx = n = 0
+        for n, tok in enumerate(splt):  # noqa
+            if tok == prefix:
+                idx = line.find(prefix, idx)
+                if idx >= begidx:
+                    break
+            prev = tok
+
+        if len(prefix) == 0:
+            n += 1
+
+    prefix_quoted = shlex.quote(prefix)
 
     script = BASH_COMPLETE_SCRIPT.format(
         source=source,
-        line=" ".join(shlex.quote(p) for p in splt),
+        line=" ".join(shlex.quote(p) for p in splt if p),
         comp_line=shlex.quote(line),
         n=n,
         cmd=shlex.quote(cmd),
@@ -375,7 +395,6 @@ def bash_completions(
     except (
         subprocess.CalledProcessError,
         FileNotFoundError,
-        UnicodeDecodeError,
         ValueError,
     ):
         return set(), 0
@@ -397,7 +416,7 @@ def bash_completions(
         strip_len += 1
 
     if "-o noquote" not in complete_stmt:
-        out, need_quotes = quote_paths(out, "", "")
+        out, need_quotes = quote_paths(out, opening_quote, closing_quote)
     if "-o nospace" in complete_stmt:
         out = set([x.rstrip() for x in out])
 
