@@ -6,12 +6,12 @@ import json
 import shlex
 import sys
 import tempfile
-import builtins
 import subprocess
 import warnings
 import functools
 import collections.abc as cabc
 
+from xonsh.built_ins import XSH
 from xonsh.lazyasd import lazyobject
 from xonsh.tools import to_bool, ensure_string
 from xonsh.platform import ON_WINDOWS, ON_CYGWIN, ON_MSYS
@@ -226,7 +226,6 @@ def foreign_shell_data(
     dryrun : bool, optional
         Whether or not to actually run and process the command.
 
-
     Returns
     -------
     env : dict
@@ -276,8 +275,8 @@ def foreign_shell_data(
         tmpfile.write(command.encode("utf8"))
         tmpfile.close()
         cmd.append(tmpfile.name)
-    if currenv is None and hasattr(builtins.__xonsh__, "env"):
-        currenv = builtins.__xonsh__.env.detype()
+    if currenv is None and XSH.env:
+        currenv = XSH.env.detype()
     elif currenv is not None:
         currenv = dict(currenv)
     try:
@@ -473,7 +472,7 @@ class ForeignShellBaseAlias(object):
         args, streaming = self._is_streaming(args)
         input = self.INPUT.format(args=" ".join(args), **self._input_kwargs())
         cmd = [self.shell] + list(self.extra_args) + ["-c", input]
-        env = builtins.__xonsh__.env
+        env = XSH.env
         denv = env.detype()
         if streaming:
             subprocess.check_call(cmd, env=denv)
@@ -700,15 +699,15 @@ def load_foreign_aliases(shells):
         A dictionary of the merged aliases.
     """
     aliases = {}
-    xonsh_aliases = builtins.aliases
+    xonsh_aliases = XSH.aliases
     for shell in shells:
         shell = ensure_shell(shell)
         _, shaliases = foreign_shell_data(**shell)
-        if not builtins.__xonsh__.env.get("FOREIGN_ALIASES_OVERRIDE"):
+        if not XSH.env.get("FOREIGN_ALIASES_OVERRIDE"):
             shaliases = {} if shaliases is None else shaliases
             for alias in set(shaliases) & set(xonsh_aliases):
                 del shaliases[alias]
-                if builtins.__xonsh__.env.get("XONSH_DEBUG") > 1:
+                if XSH.env.get("XONSH_DEBUG") >= 1:
                     print(
                         "aliases: ignoring alias {!r} of shell {!r} "
                         "which tries to override xonsh alias."
