@@ -208,23 +208,21 @@ def call_tip(oinfo, format_call=True):
     Parameters
     ----------
     oinfo : dict
-
     format_call : bool, optional
-      If True, the call line is formatted and returned as a string.  If not, a
-      tuple of (name, argspec) is returned.
+        If True, the call line is formatted and returned as a string.  If not, a
+        tuple of (name, argspec) is returned.
 
     Returns
     -------
     call_info : None, str or (str, dict) tuple.
-      When format_call is True, the whole call information is formatted as a
-      single string.  Otherwise, the object's name and its argspec dict are
-      returned.  If no call information is available, None is returned.
-
+        When format_call is True, the whole call information is formatted as a
+        single string.  Otherwise, the object's name and its argspec dict are
+        returned.  If no call information is available, None is returned.
     docstring : str or None
-      The most relevant docstring for calling purposes is returned, if
-      available.  The priority is: call docstring for callable instances, then
-      constructor docstring for classes, then main object's docstring otherwise
-      (regular functions).
+        The most relevant docstring for calling purposes is returned, if
+        available.  The priority is: call docstring for callable instances, then
+        constructor docstring for classes, then main object's docstring otherwise
+        (regular functions).
     """
     # Get call definition
     argspec = oinfo.get("argspec")
@@ -269,7 +267,7 @@ def find_file(obj):
     Returns
     -------
     fname : str
-      The absolute path to the file where the object was defined.
+        The absolute path to the file where the object was defined.
     """
     # get source if obj was decorated with @decorator
     if safe_hasattr(obj, "__wrapped__"):
@@ -287,7 +285,7 @@ def find_file(obj):
             except TypeError:
                 # Can happen for builtins
                 pass
-    except:  # pylint:disable=bare-except
+    except Exception:
         pass
     return cast_unicode(fname)
 
@@ -306,7 +304,7 @@ def find_source_lines(obj):
     Returns
     -------
     lineno : int
-      The line number where the object definition starts.
+        The line number where the object definition starts.
     """
     # get source if obj was decorated with @decorator
     if safe_hasattr(obj, "__wrapped__"):
@@ -321,7 +319,7 @@ def find_source_lines(obj):
                 lineno = inspect.getsourcelines(obj.__class__)[1]
             else:
                 lineno = None
-    except:  # pylint:disable=bare-except
+    except Exception:
         return None
 
     return lineno
@@ -342,7 +340,7 @@ class Inspector(object):
         try:
             hdef = oname + str(inspect.signature(obj))
             return cast_unicode(hdef)
-        except:  # pylint:disable=bare-except
+        except Exception:
             return None
 
     def noinfo(self, msg, oname):
@@ -395,7 +393,7 @@ class Inspector(object):
             if init_ds is not None:
                 lines.append(head("Init docstring:"))
                 lines.append(indent(init_ds))
-        elif hasattr(obj, "__call__"):
+        elif callable(obj):
             call_ds = getdoc(obj.__call__)
             if call_ds:
                 lines.append(head("Call docstring:"))
@@ -412,7 +410,7 @@ class Inspector(object):
         linecache.checkcache()
         try:
             src = getsource(obj)
-        except:  # pylint:disable=bare-except
+        except Exception:
             self.noinfo("source", oname)
         else:
             print(src)
@@ -445,9 +443,9 @@ class Inspector(object):
         Parameters
         ----------
         fields : list
-          A list of 2-tuples: (field_title, field_content)
+            A list of 2-tuples: (field_title, field_content)
         title_width : int
-          How many characters to pad titles to. Default to longest title.
+            How many characters to pad titles to. Default to longest title.
         """
         out = []
         if title_width == 0:
@@ -469,9 +467,9 @@ class Inspector(object):
         Parameters
         ----------
         fields : list
-          A list of 2-tuples: (field_title, field_content)
+            A list of 2-tuples: (field_title, field_content)
         title_width : int
-          How many characters to pad titles to. Default to longest title.
+            How many characters to pad titles to. Default to longest title.
         """
         out = []
         if title_width == 0:
@@ -499,9 +497,9 @@ class Inspector(object):
         Parameters
         ----------
         fields : list
-          A list of 2-tuples: (field_title, field_content)
+            A list of 2-tuples: (field_title, field_content)
         title_width : int
-          How many characters to pad titles to. Default to longest title.
+            How many characters to pad titles to. Default to longest title.
         """
         if HAS_PYGMENTS:
             rtn = self._format_fields_tokens(fields, title_width=title_width)
@@ -607,7 +605,7 @@ class Inspector(object):
             if not callable(obj):
                 if len(obj) >= 2 and isinstance(obj[1], str):
                     ds = "Alias to the system command:\n  {0}".format(obj[1])
-                else:  # pylint:disable=bare-except
+                else:
                     ds = "Alias: " + str(obj)
             else:
                 ds = "Alias to " + str(obj)
@@ -635,7 +633,7 @@ class Inspector(object):
         try:
             bclass = obj.__class__
             out["base_class"] = str(bclass)
-        except:  # pylint:disable=bare-except
+        except Exception:
             pass
 
         # String form, but snip if too long in ? form (full in ??)
@@ -649,7 +647,7 @@ class Inspector(object):
                         q.strip() for q in ostr.split("\n")
                     )
                 out[str_head] = ostr
-            except:  # pylint:disable=bare-except
+            except Exception:
                 pass
 
         if ospace:
@@ -658,7 +656,7 @@ class Inspector(object):
         # Length (for strings and lists)
         try:
             out["length"] = str(len(obj))
-        except:  # pylint:disable=bare-except
+        except Exception:
             pass
 
         # Filename where object was defined
@@ -738,8 +736,8 @@ class Inspector(object):
             # objects which use instance-customized docstrings.
             if ds:
                 try:
-                    cls = getattr(obj, "__class__")
-                except:  # pylint:disable=bare-except
+                    cls = obj.__class__
+                except Exception:
                     class_ds = None
                 else:
                     class_ds = getdoc(cls)
