@@ -5,17 +5,19 @@ import sys
 import time
 import ctypes
 import signal
-import builtins
 import subprocess
 import collections
 import typing as tp
 
+from xonsh.built_ins import XSH
+from xonsh.cli_utils import Annotated, Arg, ArgParserAlias
+from xonsh.completers.tools import RichCompletion
 from xonsh.lazyasd import LazyObject
 from xonsh.platform import FD_STDERR, ON_DARWIN, ON_WINDOWS, ON_CYGWIN, ON_MSYS, LIBC
 from xonsh.tools import unthreadable
 
-
-tasks = LazyObject(collections.deque, globals(), "tasks")
+# there is not much cost initing deque
+tasks: tp.Deque[int] = collections.deque()
 # Track time stamp of last exit command, so that two consecutive attempts to
 # exit can kill all jobs and exit.
 _last_exit_time: tp.Optional[float] = None
@@ -111,6 +113,7 @@ else:
 
     def _continue(job):
         _send_signal(job, signal.SIGCONT)
+        job["status"] = "running"
 
     def _kill(job):
         _send_signal(job, signal.SIGKILL)
@@ -241,7 +244,7 @@ def _safe_wait_for_active_job(last_task=None, backgrounded=False):
 
 
 def get_next_task():
-    """ Get the next active task and put it on top of the queue"""
+    """Get the next active task and put it on top of the queue"""
     _clear_dead_jobs()
     selected_task = None
     for tid in tasks:
@@ -257,7 +260,7 @@ def get_next_task():
 
 
 def get_task(tid):
-    return builtins.__xonsh__.all_jobs[tid]
+    return XSH.all_jobs[tid]
 
 
 def _clear_dead_jobs():
@@ -268,29 +271,34 @@ def _clear_dead_jobs():
             to_remove.add(tid)
     for job in to_remove:
         tasks.remove(job)
-        del builtins.__xonsh__.all_jobs[job]
+        del XSH.all_jobs[job]
+
+
+def format_job_string(num: int) -> str:
+    try:
+        job = XSH.all_jobs[num]
+    except KeyError:
+        return ""
+    pos = "+" if tasks[0] == num else "-" if tasks[1] == num else " "
+    status = job["status"]
+    cmd = " ".join([" ".join(i) if isinstance(i, list) else i for i in job["cmds"]])
+    pid = job["pids"][-1]
+    bg = " &" if job["bg"] else ""
+    return "[{}]{} {}: {}{} ({})".format(num, pos, status, cmd, bg, pid)
 
 
 def print_one_job(num, outfile=sys.stdout):
     """Print a line describing job number ``num``."""
-    try:
-        job = builtins.__xonsh__.all_jobs[num]
-    except KeyError:
-        return
-    pos = "+" if tasks[0] == num else "-" if tasks[1] == num else " "
-    status = job["status"]
-    cmd = [" ".join(i) if isinstance(i, list) else i for i in job["cmds"]]
-    cmd = " ".join(cmd)
-    pid = job["pids"][-1]
-    bg = " &" if job["bg"] else ""
-    print("[{}]{} {}: {}{} ({})".format(num, pos, status, cmd, bg, pid), file=outfile)
+    info = format_job_string(num)
+    if info:
+        print(info, file=outfile)
 
 
 def get_next_job_number():
     """Get the lowest available unique job number (for the next job created)."""
     _clear_dead_jobs()
     i = 1
-    while i in builtins.__xonsh__.all_jobs:
+    while i in XSH.all_jobs:
         i += 1
     return i
 
@@ -301,8 +309,8 @@ def add_job(info):
     info["started"] = time.time()
     info["status"] = "running"
     tasks.appendleft(num)
-    builtins.__xonsh__.all_jobs[num] = info
-    if info["bg"] and builtins.__xonsh__.env.get("XONSH_INTERACTIVE"):
+    XSH.all_jobs[num] = info
+    if info["bg"] and XSH.env.get("XONSH_INTERACTIVE"):
         print_one_job(num)
 
 
@@ -315,12 +323,12 @@ def clean_jobs():
     warning if any exist, and return False. Otherwise, return True.
     """
     jobs_clean = True
-    if builtins.__xonsh__.env["XONSH_INTERACTIVE"]:
+    if XSH.env["XONSH_INTERACTIVE"]:
         _clear_dead_jobs()
 
-        if builtins.__xonsh__.all_jobs:
+        if XSH.all_jobs:
             global _last_exit_time
-            hist = builtins.__xonsh__.history
+            hist = XSH.history
             if hist is not None and len(hist.tss) > 0:
                 last_cmd_start = hist.tss[-1][0]
             else:
@@ -333,12 +341,12 @@ def clean_jobs():
                 # unfinished jobs in this case.
                 kill_all_jobs()
             else:
-                if len(builtins.__xonsh__.all_jobs) > 1:
+                if len(XSH.all_jobs) > 1:
                     msg = "there are unfinished jobs"
                 else:
                     msg = "there is an unfinished job"
 
-                if builtins.__xonsh__.env["SHELL_TYPE"] != "prompt_toolkit":
+                if XSH.env["SHELL_TYPE"] != "prompt_toolkit":
                     # The Ctrl+D binding for prompt_toolkit already inserts a
                     # newline
                     print()
@@ -363,7 +371,7 @@ def kill_all_jobs():
     Send SIGKILL to all child processes (called when exiting xonsh).
     """
     _clear_dead_jobs()
-    for job in builtins.__xonsh__.all_jobs.values():
+    for job in XSH.all_jobs.values():
         _kill(job)
 
 
@@ -400,7 +408,7 @@ def resume_job(args, wording):
         except (ValueError, IndexError):
             return "", "Invalid job: {}\n".format(args[0])
 
-        if tid not in builtins.__xonsh__.all_jobs:
+        if tid not in XSH.all_jobs:
             return "", "Invalid job: {}\n".format(args[0])
     else:
         return "", "{} expects 0 or 1 arguments, not {}\n".format(wording, len(args))
@@ -412,7 +420,7 @@ def resume_job(args, wording):
     job = get_task(tid)
     job["bg"] = False
     job["status"] = "running"
-    if builtins.__xonsh__.env.get("XONSH_INTERACTIVE"):
+    if XSH.env.get("XONSH_INTERACTIVE"):
         print_one_job(tid)
     pipeline = job["pipeline"]
     pipeline.resume(job)
@@ -443,3 +451,64 @@ def bg(args, stdin=None):
         _continue(curtask)
     else:
         return res
+
+
+def job_id_completer(xsh, **_):
+    """Return currently running jobs ids"""
+    for job_id in xsh.all_jobs:
+        yield RichCompletion(str(job_id), description=format_job_string(job_id))
+
+
+def disown_fn(
+    job_ids: Annotated[
+        tp.Sequence[int], Arg(type=int, nargs="*", completer=job_id_completer)
+    ],
+    force_auto_continue=False,
+):
+    """Remove the specified jobs from the job table; the shell will no longer
+    report their status, and will not complain if you try to exit an
+    interactive shell with them running or stopped.
+
+    If the jobs are currently stopped and the $AUTO_CONTINUE option is not set
+    ($AUTO_CONTINUE = False), a warning is printed containing information about
+    how to make them continue after they have been disowned.
+
+    Parameters
+    ----------
+    job_ids
+        Jobs to act on or none to disown the current job
+    force_auto_continue : -c, --continue
+        Automatically continue stopped jobs when they are disowned, equivalent to setting $AUTO_CONTINUE=True
+    """
+
+    if len(tasks) == 0:
+        return "", "There are no active jobs"
+
+    messages = []
+    # if args.job_ids is empty, use the active task
+    for tid in job_ids or [tasks[0]]:
+        try:
+            current_task = get_task(tid)
+        except KeyError:
+            return "", f"'{tid}' is not a valid job ID"
+
+        auto_cont = XSH.env.get("AUTO_CONTINUE", False)
+        if auto_cont or force_auto_continue:
+            _continue(current_task)
+        elif current_task["status"] == "stopped":
+            messages.append(
+                f"warning: job is suspended, use "
+                f"'kill -CONT -{current_task['pids'][-1]}' "
+                f"to resume\n"
+            )
+
+        # Stop tracking this task
+        tasks.remove(tid)
+        del XSH.all_jobs[tid]
+        messages.append(f"Removed job {tid} ({current_task['status']})")
+
+    if messages:
+        return "".join(messages)
+
+
+disown = ArgParserAlias(prog="disown", func=disown_fn, has_args=True)
