@@ -25,7 +25,7 @@ from xonsh.lazyimps import pygments, pyghooks
 from xonsh.imphooks import install_import_hooks
 from xonsh.events import events
 from xonsh.environ import xonshrc_context, make_args_env
-from xonsh.built_ins import XonshSession, load_builtins
+from xonsh.built_ins import XSH
 
 import xonsh.procs.pipelines as xpp
 
@@ -108,14 +108,14 @@ def get_setproctitle():
 
 
 def path_argument(s):
-    """Return a path only if the path is actually legal
+    """Return a path only if the path is actually legal (file or directory)
 
     This is very similar to argparse.FileType, except that it doesn't return
     an open file handle, but rather simply validates the path."""
 
     s = os.path.abspath(os.path.expanduser(s))
-    if not os.path.isfile(s):
-        msg = "{0!r} must be a valid path to a file".format(s)
+    if not os.path.exists(s):
+        msg = "{0!r} must be a valid path to a file or directory".format(s)
         raise argparse.ArgumentTypeError(msg)
     return s
 
@@ -161,17 +161,10 @@ def parser():
         action="store_true",
         default=False,
     )
-    p.add_argument(
-        "--config-path",
-        help=argparse.SUPPRESS,
-        dest="config_path",
-        default=None,
-        type=path_argument,
-    )
     p.add_argument(
         "--rc",
         help="The xonshrc files to load, these may be either xonsh "
-        "files or JSON-based static configuration files.",
+        "files or directories containing xonsh files",
         dest="rc",
         nargs="+",
         type=path_argument,
@@ -179,7 +172,7 @@ def parser():
     )
     p.add_argument(
         "--no-rc",
-        help="Do not load the .xonshrc files.",
+        help="Do not load any xonsh RC files.",
         dest="norc",
         action="store_true",
         default=False,
@@ -251,7 +244,7 @@ def _pprint_displayhook(value):
     if isinstance(value, xpp.HiddenCommandPipeline):
         builtins._ = value
         return
-    env = builtins.__xonsh__.env
+    env = XSH.env
     printed_val = None
     if env.get("PRETTY_PRINT_RESULTS"):
         printed_val = pretty(value)
@@ -274,10 +267,12 @@ class XonshMode(enum.Enum):
     interactive = 3
 
 
-def start_services(shell_kwargs, args):
+def start_services(shell_kwargs, args, pre_env=None):
     """Starts up the essential services in the proper order.
     This returns the environment instance as a convenience.
     """
+    if pre_env is None:
+        pre_env = {}
     install_import_hooks()
     # create execer, which loads builtins
     ctx = shell_kwargs.get("ctx", {})
@@ -292,21 +287,39 @@ def start_services(shell_kwargs, args):
     events.on_timingprobe.fire(name="post_execer_init")
     # load rc files
     login = shell_kwargs.get("login", True)
-    env = builtins.__xonsh__.env
-    rc = shell_kwargs.get("rc", None)
-    rc = env.get("XONSHRC") if rc is None else rc
-    if (
+    rc_cli = shell_kwargs.get("rc")
+    env = XSH.env
+    for k, v in pre_env.items():
+        env[k] = v
+
+    # determine which RC files to load, including whether any RC directories
+    # should be scanned for such files
+    if shell_kwargs.get("norc") or (
         args.mode != XonshMode.interactive
         and not args.force_interactive
         and not args.login
     ):
-        #  Don't load xonshrc if not interactive shell
-        rc = None
+        # if --no-rc was passed, or we're not in an interactive shell and
+        # interactive mode was not forced, then disable loading RC files and dirs
+        rc = ()
+        rcd = ()
+    elif rc_cli:
+        # if an explicit --rc was passed, then we should load only that RC
+        # file, and nothing else (ignore both XONSHRC and XONSHRC_DIR)
+        rc = [r for r in rc_cli if os.path.isfile(r)]
+        rcd = [r for r in rc_cli if os.path.isdir(r)]
+    else:
+        # otherwise, get the RC files from XONSHRC, and RC dirs from XONSHRC_DIR
+        rc = env.get("XONSHRC")
+        rcd = env.get("XONSHRC_DIR")
+
     events.on_pre_rc.fire()
-    xonshrc_context(rcfiles=rc, execer=execer, ctx=ctx, env=env, login=login)
+    XSH.rc_files = xonshrc_context(
+        rcfiles=rc, rcdirs=rcd, execer=execer, ctx=ctx, env=env, login=login
+    )
     events.on_post_rc.fire()
     # create shell
-    builtins.__xonsh__.shell = Shell(execer=execer, **shell_kwargs)
+    XSH.shell = Shell(execer=execer, **shell_kwargs)
     ctx["__name__"] = "__main__"
     return env
 
@@ -315,7 +328,6 @@ def premain(argv=None):
     """Setup for main xonsh entry point. Returns parsed arguments."""
     if argv is None:
         argv = sys.argv[1:]
-    builtins.__xonsh__ = XonshSession()
     setup_timings(argv)
     setproctitle = get_setproctitle()
     if setproctitle is not None:
@@ -330,16 +342,16 @@ def premain(argv=None):
         "login": False,
         "scriptcache": args.scriptcache,
         "cacheall": args.cacheall,
-        "ctx": builtins.__xonsh__.ctx,
+        "ctx": XSH.ctx,
     }
     if args.login or sys.argv[0].startswith("-"):
         args.login = True
         shell_kwargs["login"] = True
     if args.norc:
-        shell_kwargs["rc"] = ()
+        shell_kwargs["norc"] = True
     elif args.rc:
         shell_kwargs["rc"] = args.rc
-    setattr(sys, "displayhook", _pprint_displayhook)
+    sys.displayhook = _pprint_displayhook
     if args.command is not None:
         args.mode = XonshMode.single_command
         shell_kwargs["shell_type"] = "none"
@@ -353,13 +365,15 @@ def premain(argv=None):
         args.mode = XonshMode.interactive
         shell_kwargs["completer"] = True
         shell_kwargs["login"] = True
-    env = start_services(shell_kwargs, args)
-    env["XONSH_LOGIN"] = shell_kwargs["login"]
+
+    pre_env = {
+        "XONSH_LOGIN": shell_kwargs["login"],
+        "XONSH_INTERACTIVE": args.force_interactive
+        or (args.mode == XonshMode.interactive),
+    }
+    env = start_services(shell_kwargs, args, pre_env=pre_env)
     if args.defines is not None:
         env.update([x.split("=", 1) for x in args.defines])
-    env["XONSH_INTERACTIVE"] = args.force_interactive or (
-        args.mode == XonshMode.interactive
-    )
     return args
 
 
@@ -434,9 +448,9 @@ def main_xonsh(args):
         signal.signal(signal.SIGTTOU, func_sig_ttin_ttou)
 
     events.on_post_init.fire()
-    env = builtins.__xonsh__.env
-    shell = builtins.__xonsh__.shell
-    history = builtins.__xonsh__.history
+    env = XSH.env
+    shell = XSH.shell
+    history = XSH.history
     exit_code = 0
 
     if shell and not env["XONSH_INTERACTIVE"]:
@@ -461,7 +475,9 @@ def main_xonsh(args):
                 events.on_post_cmdloop.fire()
         elif args.mode == XonshMode.single_command:
             # run a single command and exit
-            run_code_with_cache(args.command.lstrip(), shell.execer, mode="single")
+            run_code_with_cache(
+                args.command.lstrip(), shell.execer, glb=shell.ctx, mode="single"
+            )
             if history is not None and history.last_cmd_rtn is not None:
                 exit_code = history.last_cmd_rtn
         elif args.mode == XonshMode.script_from_file:
@@ -492,7 +508,7 @@ def main_xonsh(args):
 
 def postmain(args=None):
     """Teardown for main xonsh entry point, accepts parsed arguments."""
-    builtins.__xonsh__.shell = None
+    XSH.shell = None
 
 
 @contextlib.contextmanager
@@ -502,7 +518,7 @@ def main_context(argv=None):
     up the shell.
     """
     args = premain(argv)
-    yield builtins.__xonsh__.shell
+    yield XSH.shell
     postmain(args)
 
 
@@ -547,13 +563,13 @@ def setup(
     # setup xonsh ctx and execer
     if not hasattr(builtins, "__xonsh__"):
         execer = Execer(xonsh_ctx=ctx)
-        builtins.__xonsh__ = XonshSession(ctx=ctx, execer=execer)
-        load_builtins(ctx=ctx, execer=execer)
-        builtins.__xonsh__.shell = Shell(execer, ctx=ctx, shell_type=shell_type)
-    builtins.__xonsh__.env.update(env)
+        XSH.load(
+            ctx=ctx, execer=execer, shell=Shell(execer, ctx=ctx, shell_type=shell_type)
+        )
+    XSH.env.update(env)
     install_import_hooks()
-    builtins.aliases.update(aliases)
+    XSH.aliases.update(aliases)
     if xontribs:
         xontribs_load(xontribs)
-    tp = builtins.__xonsh__.commands_cache.threadable_predictors
+    tp = XSH.commands_cache.threadable_predictors
     tp.update(threadable_predictors)
