@@ -10,8 +10,6 @@ class Parser(ThreeSixParser):
 
     def __init__(
         self,
-        lexer_optimize=True,
-        lexer_table="xonsh.lexer_table",
         yacc_optimize=True,
         yacc_table="xonsh.parser_table",
         yacc_debug=False,
@@ -19,10 +17,6 @@ class Parser(ThreeSixParser):
     ):
         """Parameters
         ----------
-        lexer_optimize : bool, optional
-            Set to false when unstable and true when lexer is stable.
-        lexer_table : str, optional
-            Lexer module used when optimized.
         yacc_optimize : bool, optional
             Set to false when unstable and true when parser is stable.
         yacc_table : str, optional
@@ -43,8 +37,6 @@ class Parser(ThreeSixParser):
         for rule in tok_rules:
             self._tok_rule(rule)
         super().__init__(
-            lexer_optimize=lexer_optimize,
-            lexer_table=lexer_table,
             yacc_optimize=yacc_optimize,
             yacc_table=yacc_table,
             yacc_debug=yacc_debug,
