@@ -3,16 +3,13 @@ compatibility layers to make use of the 'best' implementation available
 on a platform.
 """
 import os
-from os import scandir
 import sys
-import ctypes
+import ctypes  # noqa
 import signal
 import pathlib
-import builtins
 import platform
 import functools
 import subprocess
-import collections
 import collections.abc as cabc
 import importlib.util
 
@@ -67,6 +64,12 @@ ON_OPENBSD = LazyBool(
     lambda: (sys.platform.startswith("openbsd")), globals(), "ON_OPENBSD"
 )
 """``True`` if on a OpenBSD operating system, else ``False``."""
+IN_APPIMAGE = LazyBool(
+    lambda: ("APPIMAGE" in os.environ and "APPDIR" in os.environ),
+    globals(),
+    "IN_APPIMAGE",
+)
+"""``True`` if in AppImage, else ``False``."""
 
 
 @lazybool
@@ -84,7 +87,17 @@ def ON_BEOS():
 @lazybool
 def ON_WSL():
     """True if we are on Windows Subsystem for Linux (WSL)"""
-    return "Microsoft" in platform.release()
+    return "microsoft" in platform.release()
+
+
+@lazybool
+def ON_WSL1():
+    return bool(ON_WSL) and not bool(ON_WSL2)
+
+
+@lazybool
+def ON_WSL2():
+    return bool(ON_WSL) and "WSL2" in platform.release()
 
 
 #
@@ -136,20 +149,13 @@ def pygments_version():
 
 @functools.lru_cache(1)
 def pygments_version_info():
-    """ Returns `pygments`'s version as tuple of integers. """
+    """Returns `pygments`'s version as tuple of integers."""
     if HAS_PYGMENTS:
         return tuple(int(x) for x in pygments_version().strip("<>+-=.").split("."))
     else:
         return None
 
 
-def use_vended_prompt_toolkit():
-    """ Unload any prompt_toolkit libraries and add vended version to sys.path """
-    for mod in (mod for mod in list(sys.modules) if mod.startswith("prompt_toolkit")):
-        del sys.modules[mod]
-    sys.path.insert(0, str(pathlib.Path(__file__).with_name("vended_ptk").resolve()))
-
-
 @functools.lru_cache(1)
 def has_prompt_toolkit():
     """Tests if the `prompt_toolkit` is available."""
@@ -170,7 +176,7 @@ def ptk_version():
 
 @functools.lru_cache(1)
 def ptk_version_info():
-    """ Returns `prompt_toolkit`'s version as tuple of integers. """
+    """Returns `prompt_toolkit`'s version as tuple of integers."""
     if has_prompt_toolkit():
         return tuple(int(x) for x in ptk_version().strip("<>+-=.").split("."))
     else:
@@ -206,10 +212,13 @@ def ptk_below_max_supported():
 
 @functools.lru_cache(1)
 def best_shell_type():
-    if builtins.__xonsh__.env.get("TERM", "") == "dumb":
+    from xonsh.built_ins import XSH
+
+    if XSH.env.get("TERM", "") == "dumb":
         return "dumb"
-    else:
+    if has_prompt_toolkit():
         return "prompt_toolkit"
+    return "readline"
 
 
 @functools.lru_cache(1)
@@ -233,6 +242,9 @@ def pathsplit(p):
     without a drive.
     """
     n = len(p)
+    if n == 0:
+        # lazy object seps does not get initialized when n is zero
+        return "", ""
     while n and p[n - 1] not in seps:
         n -= 1
     pre = p[:n]
@@ -263,6 +275,7 @@ def windows_expanduser(path):
     exists. This restricts expanding the '~' if it is not followed by a
     separator. That is only '~/' and '~\' are expanded.
     """
+    path = str(path)
     if not path.startswith("~"):
         return path
     elif len(path) < 2 or path[1] in seps:
@@ -358,8 +371,10 @@ def windows_bash_command():
     """Determines the command for Bash on windows."""
     # Check that bash is on path otherwise try the default directory
     # used by Git for windows
+    from xonsh.built_ins import XSH
+
     wbc = "bash"
-    cmd_cache = builtins.__xonsh__.commands_cache
+    cmd_cache = XSH.commands_cache
     bash_on_path = cmd_cache.lazy_locate_binary("bash", ignore_alias=True)
     if bash_on_path:
         try:
