@@ -5,7 +5,6 @@ import io
 import sys
 import time
 import signal
-import builtins
 import threading
 import subprocess
 
@@ -13,6 +12,7 @@ import xonsh.lazyasd as xl
 import xonsh.tools as xt
 import xonsh.platform as xp
 import xonsh.jobs as xj
+from xonsh.built_ins import XSH
 
 from xonsh.procs.readers import NonBlockingFDReader, ConsoleParallelReader, safe_fdclose
 
@@ -83,7 +83,7 @@ def update_fg_process_group(pipeline_group, background):
         return False
     if not xp.ON_POSIX:
         return False
-    env = builtins.__xonsh__.env
+    env = XSH.env
     if not env.get("XONSH_INTERACTIVE"):
         return False
     return xj.give_terminal_to(pipeline_group)
@@ -184,9 +184,28 @@ class CommandPipeline:
     def __bool__(self):
         return self.returncode == 0
 
+    def __int__(self):
+        return self.returncode
+
+    def __hash__(self):
+        return hash(self.returncode)
+
+    def __str__(self):
+        self.end()
+        return self.output
+
     def __len__(self):
         return len(self.procs)
 
+    def __eq__(self, other):
+        if isinstance(other, int):
+            return self.returncode == other
+        elif isinstance(other, str):
+            return str(self) == other
+        raise Exception(
+            f"CommandPipeline doesn't support comparing with {type(other)}."
+        )
+
     def __iter__(self):
         """Iterates through stdout and returns the lines, converting to
         strings and universal newlines if needed.
@@ -205,7 +224,7 @@ class CommandPipeline:
         proc = self.proc
         if proc is None:
             return
-        timeout = builtins.__xonsh__.env.get("XONSH_PROC_FREQUENCY")
+        timeout = XSH.env.get("XONSH_PROC_FREQUENCY")
         # get the correct stdout
         stdout = proc.stdout
         if (
@@ -328,7 +347,7 @@ class CommandPipeline:
         yields each line. This may optionally accept lines (in bytes) to iterate
         over, in which case it does not call iterraw().
         """
-        env = builtins.__xonsh__.env
+        env = XSH.env
         enc = env.get("XONSH_ENCODING")
         err = env.get("XONSH_ENCODING_ERRORS")
         lines = self.lines
@@ -368,7 +387,7 @@ class CommandPipeline:
         """Streams lines to sys.stderr and the errors attribute."""
         if not lines:
             return
-        env = builtins.__xonsh__.env
+        env = XSH.env
         enc = env.get("XONSH_ENCODING")
         err = env.get("XONSH_ENCODING_ERRORS")
         b = b"".join(lines)
@@ -377,18 +396,22 @@ class CommandPipeline:
         if self.stderr_postfix:
             b += self.stderr_postfix
         stderr_has_buffer = hasattr(sys.stderr, "buffer")
-        # write bytes to std stream
-        if stderr_has_buffer:
-            sys.stderr.buffer.write(b)
-        else:
-            sys.stderr.write(b.decode(encoding=enc, errors=err))
-        sys.stderr.flush()
+        show_stderr = self.captured != "object" or env.get(
+            "XONSH_SUBPROC_CAPTURED_PRINT_STDERR", True
+        )
+        if show_stderr:
+            # write bytes to std stream
+            if stderr_has_buffer:
+                sys.stderr.buffer.write(b)
+            else:
+                sys.stderr.write(b.decode(encoding=enc, errors=err))
+            sys.stderr.flush()
         # save the raw bytes
         self._raw_error = b
         # do some munging of the line before we save it to the attr
         b = b.replace(b"\r\n", b"\n").replace(b"\r", b"\n")
         b = RE_HIDE_ESCAPE.sub(b"", b)
-        env = builtins.__xonsh__.env
+        env = XSH.env
         s = b.decode(
             encoding=env.get("XONSH_ENCODING"), errors=env.get("XONSH_ENCODING_ERRORS")
         )
@@ -402,8 +425,8 @@ class CommandPipeline:
         """Decode bytes into a str and apply universal newlines as needed."""
         if not b:
             return ""
-        if isinstance(b, bytes):
-            env = builtins.__xonsh__.env
+        if isinstance(b, (bytes, bytearray)):
+            env = XSH.env
             s = b.decode(
                 encoding=env.get("XONSH_ENCODING"),
                 errors=env.get("XONSH_ENCODING_ERRORS"),
@@ -455,12 +478,12 @@ class CommandPipeline:
             return
         if xj.give_terminal_to(pgid):  # if gave term succeed
             self.term_pgid = pgid
-            if builtins.__xonsh__.shell is not None:
+            if XSH.shell is not None:
                 # restoring sanity could probably be called whenever we return
                 # control to the shell. But it only seems to matter after a
                 # ^Z event. This *has* to be called after we give the terminal
                 # back to the shell.
-                builtins.__xonsh__.shell.shell.restore_tty_sanity()
+                XSH.shell.shell.restore_tty_sanity()
 
     def resume(self, job, tee_output=True):
         self.ended = False
@@ -557,7 +580,7 @@ class CommandPipeline:
 
     def _apply_to_history(self):
         """Applies the results to the current history object."""
-        hist = builtins.__xonsh__.history
+        hist = XSH.history
         if hist is not None:
             hist.last_cmd_rtn = 1 if self.proc is None else self.proc.returncode
 
@@ -565,11 +588,7 @@ class CommandPipeline:
         """Raises a subprocess error, if we are supposed to."""
         spec = self.spec
         rtn = self.returncode
-        if (
-            rtn is not None
-            and rtn != 0
-            and builtins.__xonsh__.env.get("RAISE_SUBPROC_ERROR")
-        ):
+        if rtn is not None and rtn != 0 and XSH.env.get("RAISE_SUBPROC_ERROR"):
             try:
                 raise subprocess.CalledProcessError(rtn, spec.args, output=self.output)
             finally:
@@ -701,7 +720,7 @@ class CommandPipeline:
         """Prefix to print in front of stderr, as bytes."""
         p = self._stderr_prefix
         if p is None:
-            env = builtins.__xonsh__.env
+            env = XSH.env
             t = env.get("XONSH_STDERR_PREFIX")
             s = xt.format_std_prepost(t, env=env)
             p = s.encode(
@@ -716,7 +735,7 @@ class CommandPipeline:
         """Postfix to print after stderr, as bytes."""
         p = self._stderr_postfix
         if p is None:
-            env = builtins.__xonsh__.env
+            env = XSH.env
             t = env.get("XONSH_STDERR_POSTFIX")
             s = xt.format_std_prepost(t, env=env)
             p = s.encode(
@@ -788,7 +807,7 @@ class PrevProcCloser(threading.Thread):
             return
         proc = pipeline.proc
         prev_end_time = None
-        timeout = builtins.__xonsh__.env.get("XONSH_PROC_FREQUENCY")
+        timeout = XSH.env.get("XONSH_PROC_FREQUENCY")
         sleeptime = min(timeout * 1000, 0.1)
         while proc.poll() is None:
             if not check_prev_done:
