@@ -5,7 +5,6 @@ import sys
 import time
 import array
 import signal
-import builtins
 import threading
 import subprocess
 
@@ -13,6 +12,7 @@ import xonsh.lazyasd as xl
 import xonsh.platform as xp
 import xonsh.tools as xt
 import xonsh.lazyimps as xli
+from xonsh.built_ins import XSH
 
 from xonsh.procs.readers import (
     BufferedFDParallelReader,
@@ -52,8 +52,11 @@ class PopenThread(threading.Thread):
 
     def __init__(self, *args, stdin=None, stdout=None, stderr=None, **kwargs):
         super().__init__()
+
+        self.daemon = True
+
         self.lock = threading.RLock()
-        env = builtins.__xonsh__.env
+        env = XSH.env
         # stdin setup
         self.orig_stdin = stdin
         if stdin is None:
@@ -115,6 +118,8 @@ class PopenThread(threading.Thread):
             self.stderr = io.BytesIO()
         self.suspended = False
         self.prevs_are_closed = False
+        # This is so the thread will use the same swapped values as the origin one.
+        self.original_swapped_values = XSH.env.get_swapped_values()
         self.start()
 
     def run(self):
@@ -122,6 +127,8 @@ class PopenThread(threading.Thread):
         and copying bytes from captured_stdout to stdout and bytes from
         captured_stderr to stderr.
         """
+        # Set the thread-local swapped values.
+        XSH.env.set_swapped_values(self.original_swapped_values)
         proc = self.proc
         spec = self._wait_and_getattr("spec")
         # get stdin and apply parallel reader if needed.
@@ -211,7 +218,7 @@ class PopenThread(threading.Thread):
         if reader is None:
             return 0
         i = -1
-        for i, chunk in enumerate(iter(reader.read_queue, b"")):
+        for i, chunk in enumerate(iter(reader.read_queue, b"")):  # noqa
             self._alt_mode_switch(chunk, writer, stdbuf)
         if i >= 0:
             writer.flush()
