@@ -12,7 +12,6 @@ import sys
 import time
 import signal
 import inspect
-import builtins
 import functools
 import threading
 import subprocess
@@ -21,6 +20,7 @@ import collections.abc as cabc
 import xonsh.tools as xt
 import xonsh.platform as xp
 import xonsh.lazyimps as xli
+from xonsh.built_ins import XSH
 
 from xonsh.procs.readers import safe_fdclose
 
@@ -214,6 +214,8 @@ class FileThreadDispatcher:
 
     def isatty(self):
         """Returns if the file descriptor for the current thread is a tty."""
+        if self.default:
+            return self.default.isatty()
         return self.handle.isatty()
 
     def readable(self):
@@ -323,7 +325,12 @@ PROXIES = (proxy_zero, proxy_one, proxy_two, proxy_three, proxy_four, proxy_five
 def partial_proxy(f):
     """Dispatches the appropriate proxy function based on the number of args."""
     numargs = 0
+
     for name, param in inspect.signature(f).parameters.items():
+        # handle *args/**kwargs signature
+        if param.kind in {param.VAR_KEYWORD, param.VAR_POSITIONAL}:
+            numargs = 6
+            break
         if (
             param.kind == param.POSITIONAL_ONLY
             or param.kind == param.POSITIONAL_OR_KEYWORD
@@ -407,7 +414,7 @@ class ProcProxyThread(threading.Thread):
         self.stdout = stdout
         self.stderr = stderr
         self.close_fds = close_fds
-        self.env = env or builtins.__xonsh__.env
+        self.env = env
         self._interrupted = False
 
         if xp.ON_WINDOWS:
@@ -444,6 +451,8 @@ class ProcProxyThread(threading.Thread):
             self.old_int_handler = signal.signal(signal.SIGINT, self._signal_int)
         # start up the proc
         super().__init__()
+        # This is so the thread will use the same swapped values as the origin one.
+        self.original_swapped_values = XSH.env.get_swapped_values()
         self.start()
 
     def __del__(self):
@@ -456,12 +465,14 @@ class ProcProxyThread(threading.Thread):
         """
         if self.f is None:
             return
+        # Set the thread-local swapped values.
+        XSH.env.set_swapped_values(self.original_swapped_values)
         spec = self._wait_and_getattr("spec")
         last_in_pipeline = spec.last_in_pipeline
         if last_in_pipeline:
             capout = spec.captured_stdout  # NOQA
             caperr = spec.captured_stderr  # NOQA
-        env = builtins.__xonsh__.env
+        env = XSH.env
         enc = env.get("XONSH_ENCODING")
         err = env.get("XONSH_ENCODING_ERRORS")
         if xp.ON_WINDOWS:
@@ -498,10 +509,16 @@ class ProcProxyThread(threading.Thread):
             sp_stderr = sys.stderr
         # run the function itself
         try:
+            alias_stack = XSH.env.get("__ALIAS_STACK", "")
+            if self.env and self.env.get("__ALIAS_NAME"):
+                alias_stack += ":" + self.env["__ALIAS_NAME"]
+
             with STDOUT_DISPATCHER.register(sp_stdout), STDERR_DISPATCHER.register(
                 sp_stderr
             ), xt.redirect_stdout(STDOUT_DISPATCHER), xt.redirect_stderr(
                 STDERR_DISPATCHER
+            ), XSH.env.swap(
+                self.env, __ALIAS_STACK=alias_stack
             ):
                 r = self.f(self.args, sp_stdin, sp_stdout, sp_stderr, spec, spec.stack)
         except SystemExit as e:
@@ -783,7 +800,7 @@ class ProcProxy:
         """
         if self.f is None:
             return 0
-        env = builtins.__xonsh__.env
+        env = XSH.env
         enc = env.get("XONSH_ENCODING")
         err = env.get("XONSH_ENCODING_ERRORS")
         spec = self._wait_and_getattr("spec")
@@ -800,7 +817,8 @@ class ProcProxy:
         stderr = self._pick_buf(self.stderr, sys.stderr, enc, err)
         # run the actual function
         try:
-            r = self.f(self.args, stdin, stdout, stderr, spec, spec.stack)
+            with XSH.env.swap(self.env):
+                r = self.f(self.args, stdin, stdout, stderr, spec, spec.stack)
         except Exception:
             xt.print_exception()
             r = 1
