@@ -3,14 +3,15 @@ import os
 import io
 import re
 import sys
+import stat
 import shlex
 import signal
 import inspect
 import pathlib
-import builtins
 import subprocess
 import contextlib
 
+from xonsh.built_ins import XSH
 import xonsh.tools as xt
 import xonsh.lazyasd as xl
 import xonsh.platform as xp
@@ -39,13 +40,19 @@ def is_app_execution_alias(fname):
     Here we try to detect if a file is an app execution alias.
     """
     fname = pathlib.Path(fname)
-    return not os.path.exists(fname) and fname.name in os.listdir(fname.parent)
+    try:
+        return fname.stat().st_reparse_tag == stat.IO_REPARSE_TAG_APPEXECLINK
+
+    # os.stat().st_reparse_tag is python 3.8+, and os.stat(app_exec_alias) throws OSError for <= 3.7
+    # so use old method as fallback
+    except (AttributeError, OSError):
+        return not os.path.exists(fname) and fname.name in os.listdir(fname.parent)
 
 
 def _is_binary(fname, limit=80):
     try:
         with open(fname, "rb") as f:
-            for i in range(limit):
+            for _ in range(limit):
                 char = f.read(1)
                 if char == b"\0":
                     return True
@@ -101,7 +108,7 @@ def get_script_subproc_command(fname, args):
         # Windows can execute various filetypes directly
         # as given in PATHEXT
         _, ext = os.path.splitext(fname)
-        if ext.upper() in builtins.__xonsh__.env.get("PATHEXT"):
+        if ext.upper() in XSH.env.get("PATHEXT"):
             return [fname] + args
     # find interpreter
     with open(fname, "rb") as f:
@@ -355,6 +362,8 @@ class SubprocSpec:
         # pure attrs
         self.args = list(cmd)
         self.alias = None
+        self.alias_name = None
+        self.alias_stack = XSH.env.get("__ALIAS_STACK", "").split(":")
         self.binary_loc = None
         self.is_proxy = False
         self.background = False
@@ -440,13 +449,13 @@ class SubprocSpec:
         event_name = self._cmd_event_name()
         self._pre_run_event_fire(event_name)
         kwargs = {n: getattr(self, n) for n in self.kwnames}
-        self.prep_env(kwargs)
-        self.prep_preexec_fn(kwargs, pipeline_group=pipeline_group)
         if callable(self.alias):
-            if "preexec_fn" in kwargs:
-                kwargs.pop("preexec_fn")
+            kwargs["env"] = self.env or {}
+            kwargs["env"]["__ALIAS_NAME"] = self.alias_name or ""
             p = self.cls(self.alias, self.cmd, **kwargs)
         else:
+            self.prep_env_subproc(kwargs)
+            self.prep_preexec_fn(kwargs, pipeline_group=pipeline_group)
             self._fix_null_cmd_bytes()
             p = self._run_binary(kwargs)
         p.spec = self
@@ -457,9 +466,16 @@ class SubprocSpec:
         return p
 
     def _run_binary(self, kwargs):
+        if not self.cmd[0]:
+            raise xt.XonshError("xonsh: subprocess mode: command is empty")
+        bufsize = 1
         try:
-            bufsize = 1
-            p = self.cls(self.cmd, bufsize=bufsize, **kwargs)
+            if xp.ON_WINDOWS and self.binary_loc is not None:
+                # launch process using full paths (https://bugs.python.org/issue8557)
+                cmd = [self.binary_loc] + self.cmd[1:]
+            else:
+                cmd = self.cmd
+            p = self.cls(cmd, bufsize=bufsize, **kwargs)
         except PermissionError:
             e = "xonsh: subprocess mode: permission denied: {0}"
             raise xt.XonshError(e.format(self.cmd[0]))
@@ -471,16 +487,16 @@ class SubprocSpec:
                         ["man", cmd0.rstrip("?")], bufsize=bufsize, **kwargs
                     )
             e = "xonsh: subprocess mode: command not found: {0}".format(cmd0)
-            env = builtins.__xonsh__.env
-            sug = xt.suggest_commands(cmd0, env, builtins.aliases)
+            env = XSH.env
+            sug = xt.suggest_commands(cmd0, env)
             if len(sug.strip()) > 0:
-                e += "\n" + xt.suggest_commands(cmd0, env, builtins.aliases)
+                e += "\n" + xt.suggest_commands(cmd0, env)
             raise xt.XonshError(e)
         return p
 
-    def prep_env(self, kwargs):
+    def prep_env_subproc(self, kwargs):
         """Prepares the environment to use in the subprocess."""
-        with builtins.__xonsh__.env.swap(self.env) as env:
+        with XSH.env.swap(self.env) as env:
             denv = env.detype()
         if xp.ON_WINDOWS:
             # Over write prompt variable as xonsh's $PROMPT does
@@ -492,9 +508,9 @@ class SubprocSpec:
         """Prepares the 'preexec_fn' keyword argument"""
         if not xp.ON_POSIX:
             return
-        if not builtins.__xonsh__.env.get("XONSH_INTERACTIVE"):
+        if not XSH.env.get("XONSH_INTERACTIVE"):
             return
-        if pipeline_group is None or xp.ON_WSL:
+        if pipeline_group is None or xp.ON_WSL1:
             # If there is no pipeline group
             # or the platform is windows subsystem for linux (WSL)
             xonsh_preexec_fn = no_pg_xonsh_preexec_fn
@@ -516,6 +532,8 @@ class SubprocSpec:
         # escape them just in case.
         cmd = self.cmd
         for i in range(len(cmd)):
+            if callable(cmd[i]):
+                raise Exception(f"The command contains callable argument: {cmd[i]}")
             cmd[i] = cmd[i].replace("\0", "\\0")
 
     def _cmd_event_name(self):
@@ -527,14 +545,14 @@ class SubprocSpec:
             return os.path.basename(self.binary_loc)
 
     def _pre_run_event_fire(self, name):
-        events = builtins.events
+        events = XSH.builtins.events
         event_name = "on_pre_spec_run_" + name
         if events.exists(event_name):
             event = getattr(events, event_name)
             event.fire(spec=self)
 
     def _post_run_event_fire(self, name, proc):
-        events = builtins.events
+        events = XSH.builtins.events
         event_name = "on_post_spec_run_" + name
         if events.exists(event_name):
             event = getattr(events, event_name)
@@ -550,6 +568,8 @@ class SubprocSpec:
         modifications and adjustments based on the actual cmd that
         was received.
         """
+        if not cmd:
+            raise xt.XonshError("xonsh: subprocess mode: command is empty")
         # modifications that do not alter cmds may come before creating instance
         spec = kls(cmd, cls=cls, **kwargs)
         # modifications that alter cmds must come after creating instance
@@ -566,7 +586,7 @@ class SubprocSpec:
         return spec
 
     def redirect_leading(self):
-        """Manage leading redirects such as with '< input.txt COMMAND'. """
+        """Manage leading redirects such as with '< input.txt COMMAND'."""
         while len(self.cmd) >= 3 and self.cmd[0] == "<":
             self.stdin = safe_open(self.cmd[1], "r")
             self.cmd = self.cmd[2:]
@@ -589,17 +609,29 @@ class SubprocSpec:
     def resolve_alias(self):
         """Sets alias in command, if applicable."""
         cmd0 = self.cmd[0]
+
+        if cmd0 in self.alias_stack:
+            # Disabling the alias resolving to prevent infinite loop in call stack
+            # and futher using binary_loc to resolve the alias name.
+            self.alias = None
+            return
+
         if callable(cmd0):
             alias = cmd0
         else:
-            alias = builtins.aliases.get(cmd0, None)
+            alias = XSH.aliases.get(cmd0, None)
+            if alias is not None:
+                self.alias_name = cmd0
         self.alias = alias
 
     def resolve_binary_loc(self):
         """Sets the binary location"""
         alias = self.alias
         if alias is None:
-            binary_loc = xenv.locate_binary(self.cmd[0])
+            cmd0 = self.cmd[0]
+            binary_loc = xenv.locate_binary(cmd0)
+            if binary_loc == cmd0 and cmd0 in self.alias_stack:
+                raise Exception(f'Recursive calls to "{cmd0}" alias.')
         elif callable(alias):
             binary_loc = None
         else:
@@ -612,12 +644,12 @@ class SubprocSpec:
             self.alias is None
             and self.binary_loc is None
             and len(self.cmd) == 1
-            and builtins.__xonsh__.env.get("AUTO_CD")
+            and XSH.env.get("AUTO_CD")
             and os.path.isdir(self.cmd[0])
         ):
             return
         self.cmd.insert(0, "cd")
-        self.alias = builtins.aliases.get("cd", None)
+        self.alias = XSH.aliases.get("cd", None)
 
     def resolve_executable_commands(self):
         """Resolve command executables, if applicable."""
@@ -648,7 +680,7 @@ class SubprocSpec:
         if not callable(alias):
             return
         self.is_proxy = True
-        env = builtins.__xonsh__.env
+        env = XSH.env
         thable = env.get("THREAD_SUBPROCS") and getattr(
             alias, "__xonsh_threadable__", True
         )
@@ -703,7 +735,7 @@ def _safe_pipe_properties(fd, use_tty=False):
 
 
 def _update_last_spec(last):
-    env = builtins.__xonsh__.env
+    env = XSH.env
     captured = last.captured
     last.last_in_pipeline = True
     if not captured:
@@ -712,9 +744,10 @@ def _update_last_spec(last):
     if callable_alias:
         pass
     else:
-        cmds_cache = builtins.__xonsh__.commands_cache
+        cmds_cache = XSH.commands_cache
         thable = (
             env.get("THREAD_SUBPROCS")
+            and (captured != "hiddenobject" or env.get("XONSH_CAPTURE_ALWAYS"))
             and cmds_cache.predict_threadable(last.args)
             and cmds_cache.predict_threadable(last.cmd)
         )
@@ -738,9 +771,9 @@ def _update_last_spec(last):
         r, w = os.pipe()
         last.stdout = safe_open(w, "wb")
         last.captured_stdout = safe_open(r, "rb")
-    elif builtins.__xonsh__.stdout_uncaptured is not None:
+    elif XSH.stdout_uncaptured is not None:
         last.universal_newlines = True
-        last.stdout = builtins.__xonsh__.stdout_uncaptured
+        last.stdout = XSH.stdout_uncaptured
         last.captured_stdout = last.stdout
     elif xp.ON_WINDOWS and not callable_alias:
         last.universal_newlines = True
@@ -756,12 +789,14 @@ def _update_last_spec(last):
     # set standard error
     if last.stderr is not None:
         pass
+    elif captured == "stdout":
+        pass
     elif captured == "object":
         r, w = os.pipe()
         last.stderr = safe_open(w, "w")
         last.captured_stderr = safe_open(r, "r")
-    elif builtins.__xonsh__.stderr_uncaptured is not None:
-        last.stderr = builtins.__xonsh__.stderr_uncaptured
+    elif XSH.stderr_uncaptured is not None:
+        last.stderr = XSH.stderr_uncaptured
         last.captured_stderr = last.stderr
     elif xp.ON_WINDOWS and not callable_alias:
         last.universal_newlines = True
@@ -811,18 +846,29 @@ def cmds_to_specs(cmds, captured=False, envs=None):
             specs[i].background = True
         else:
             raise xt.XonshError(f"unrecognized redirect {redirect!r}")
+
     # Apply boundary conditions
+    if not XSH.env.get("XONSH_CAPTURE_ALWAYS"):
+        # Make sure sub-specs are always captured.
+        # I.e. ![some_alias | grep x] $(some_alias)
+        specs_to_capture = specs if captured in STDOUT_CAPTURE_KINDS else specs[:-1]
+        for spec in specs_to_capture:
+            if spec.env is None:
+                spec.env = {"XONSH_CAPTURE_ALWAYS": True}
+            else:
+                spec.env.setdefault("XONSH_CAPTURE_ALWAYS", True)
+
     _update_last_spec(specs[-1])
     return specs
 
 
 def _should_set_title(captured=False):
-    env = builtins.__xonsh__.env
+    env = XSH.env
     return (
         env.get("XONSH_INTERACTIVE")
         and not env.get("XONSH_STORE_STDOUT")
         and captured not in STDOUT_CAPTURE_KINDS
-        and builtins.__xonsh__.shell is not None
+        and XSH.shell is not None
     )
 
 
@@ -839,8 +885,12 @@ def run_subproc(cmds, captured=False, envs=None):
 
     Lastly, the captured argument affects only the last real command.
     """
-    if builtins.__xonsh__.env.get("XONSH_TRACE_SUBPROC"):
-        print(f"TRACE SUBPROC: {cmds}", file=sys.stderr)
+    if XSH.env.get("XONSH_TRACE_SUBPROC", False):
+        tracer = XSH.env.get("XONSH_TRACE_SUBPROC_FUNC")
+        if callable(tracer):
+            tracer(cmds, captured=captured)
+        else:
+            print(f"TRACE SUBPROC: {cmds}, captured={captured}", file=sys.stderr)
 
     specs = cmds_to_specs(cmds, captured=captured, envs=envs)
     captured = specs[-1].captured
@@ -863,22 +913,21 @@ def run_subproc(cmds, captured=False, envs=None):
         )
     if _should_set_title(captured=captured):
         # set title here to get currently executing command
-        pause_call_resume(proc, builtins.__xonsh__.shell.settitle)
+        pause_call_resume(proc, XSH.shell.settitle)
     else:
         # for some reason, some programs are in a stopped state when the flow
         # reaches this point, hence a SIGCONT should be sent to `proc` to make
         # sure that the shell doesn't hang. This `pause_call_resume` invocation
         # does this
         pause_call_resume(proc, int)
-    # create command or return if backgrounding.
-    if background:
+    # now figure out what we should return
+    if captured == "object":
+        return command  # object can be returned even if backgrounding
+    elif background:
         return
-    # now figure out what we should return.
-    if captured == "stdout":
+    elif captured == "stdout":
         command.end()
         return command.output
-    elif captured == "object":
-        return command
     elif captured == "hiddenobject":
         command.end()
         return command
