@@ -15,12 +15,12 @@ import sys
 import cmd
 import select
 import shutil
-import builtins
 import importlib
 import threading
 import collections
 
 import xonsh.completers.tools as xct
+from xonsh.built_ins import XSH
 from xonsh.lazyasd import LazyObject, lazyobject
 from xonsh.base_shell import BaseShell
 from xonsh.ansi_colors import (
@@ -29,7 +29,13 @@ from xonsh.ansi_colors import (
     ansi_color_style,
 )
 from xonsh.prompt.base import multiline_prompt
-from xonsh.tools import print_exception, to_bool, columnize, carriage_return
+from xonsh.tools import (
+    print_exception,
+    to_bool,
+    columnize,
+    carriage_return,
+    ends_with_colon_token,
+)
 from xonsh.platform import (
     ON_WINDOWS,
     ON_CYGWIN,
@@ -103,7 +109,7 @@ def setup_readline():
         except Exception:
             pass
         RL_CAN_RESIZE = hasattr(lib, "rl_reset_screen_size")
-    env = builtins.__xonsh__.env
+    env = XSH.env
     # reads in history
     readline.set_history_length(-1)
     ReadlineHistoryAdder()
@@ -155,6 +161,13 @@ def setup_readline():
         except Exception:
             # this seems to fail with libedit
             print_exception("xonsh: could not load readline default init file.")
+
+    # Protection against paste jacking (issue #1154)
+    # This must be set after the init file is loaded since read_init_file()
+    # automatically disables bracketed paste
+    # (https://github.com/python/cpython/pull/24108)
+    readline.parse_and_bind("set enable-bracketed-paste on")
+
     # properly reset input typed before the first prompt
     readline.set_startup_hook(carriage_return)
 
@@ -170,7 +183,7 @@ def teardown_readline():
 def _rebind_case_sensitive_completions():
     # handle case sensitive, see Github issue #1342 for details
     global _RL_PREV_CASE_SENSITIVE_COMPLETIONS
-    env = builtins.__xonsh__.env
+    env = XSH.env
     case_sensitive = env.get("CASE_SENSITIVE_COMPLETIONS")
     if case_sensitive is _RL_PREV_CASE_SENSITIVE_COMPLETIONS:
         return
@@ -221,7 +234,7 @@ def rl_completion_query_items(val=None):
     if RL_COMPLETION_QUERY_ITEMS is None:
         return
     if val is None:
-        val = builtins.__xonsh__.env.get("COMPLETION_QUERY_LIMIT")
+        val = XSH.env.get("COMPLETION_QUERY_LIMIT")
     RL_COMPLETION_QUERY_ITEMS.value = val
 
 
@@ -240,7 +253,7 @@ def rl_variable_value(variable):
 
         RL_VARIABLE_VALUE = RL_LIB.rl_variable_value
         RL_VARIABLE_VALUE.restype = ctypes.c_char_p
-    env = builtins.__xonsh__.env
+    env = XSH.env
     enc, errors = env.get("XONSH_ENCODING"), env.get("XONSH_ENCODING_ERRORS")
     if isinstance(variable, str):
         variable = variable.encode(encoding=enc, errors=errors)
@@ -307,7 +320,11 @@ class ReadlineShell(BaseShell, cmd.Cmd):
     """The readline based xonsh shell."""
 
     def __init__(self, completekey="tab", stdin=None, stdout=None, **kwargs):
-        super().__init__(completekey=completekey, stdin=stdin, stdout=stdout, **kwargs)
+        BaseShell.__init__(self, **kwargs)
+        # super() doesn't pass the stdin/stdout to Cmd's init method correctly.
+        # so calling it explicitly
+        cmd.Cmd.__init__(self, completekey=completekey, stdin=stdin, stdout=stdout)
+
         setup_readline()
         self._current_indent = ""
         self._current_prompt = ""
@@ -386,7 +403,7 @@ class ReadlineShell(BaseShell, cmd.Cmd):
         """
         if os.path.commonprefix([c[loc:] for c in completions]):
             return 1
-        elif len(completions) <= builtins.__xonsh__.env.get("COMPLETION_QUERY_LIMIT"):
+        elif len(completions) <= XSH.env.get("COMPLETION_QUERY_LIMIT"):
             return 2
         msg = "\nDisplay all {} possibilities? ".format(len(completions))
         msg += "({GREEN}y{RESET} or {RED}n{RESET})"
@@ -426,8 +443,15 @@ class ReadlineShell(BaseShell, cmd.Cmd):
         rl_completion_suppress_append()  # this needs to be called each time
         _rebind_case_sensitive_completions()
         rl_completion_query_items(val=999999999)
+        prev_text = "".join(self.buffer)
         completions, plen = self.completer.complete(
-            prefix, line, begidx, endidx, ctx=self.ctx
+            prefix,
+            line,
+            begidx,
+            endidx,
+            ctx=self.ctx,
+            multiline_text=prev_text + line,
+            cursor_index=len(prev_text) + endidx,
         )
         rtn_completions = _render_completions(completions, prefix, plen)
 
@@ -484,13 +508,13 @@ class ReadlineShell(BaseShell, cmd.Cmd):
             if len(line.strip()) == 0:
                 readline.set_pre_input_hook(None)
                 self._current_indent = ""
-            elif line.rstrip()[-1] == ":":
+            elif ends_with_colon_token(line):
                 ind = line[: len(line) - len(line.lstrip())]
-                ind += builtins.__xonsh__.env.get("INDENT")
+                ind += XSH.env.get("INDENT")
                 readline.set_pre_input_hook(_insert_text_func(ind, readline))
                 self._current_indent = ind
             elif line.split(maxsplit=1)[0] in DEDENT_TOKENS:
-                env = builtins.__xonsh__.env
+                env = XSH.env
                 ind = self._current_indent[: -len(env.get("INDENT"))]
                 readline.set_pre_input_hook(_insert_text_func(ind, readline))
                 self._current_indent = ind
@@ -513,16 +537,16 @@ class ReadlineShell(BaseShell, cmd.Cmd):
         (C) Python Software Foundation, 2015.
         """
         self.preloop()
-        if self.use_rawinput and self.completekey:
-            try:
-                import readline
+        try:
+            import readline
 
+            if self.use_rawinput and self.completekey:
                 self.old_completer = readline.get_completer()
                 readline.set_completer(self.complete)
                 readline.parse_and_bind(self.completekey + ": complete")
-                have_readline = True
-            except ImportError:
-                have_readline = False
+            have_readline = True
+        except ImportError:
+            have_readline = False
         try:
             if intro is not None:
                 self.intro = intro
@@ -544,7 +568,7 @@ class ReadlineShell(BaseShell, cmd.Cmd):
                     try:
                         line = self.singleline()
                     except EOFError:
-                        if builtins.__xonsh__.env.get("IGNOREEOF"):
+                        if XSH.env.get("IGNOREEOF"):
                             self.stdout.write('Use "exit" to leave the shell.' "\n")
                             line = ""
                         else:
@@ -582,11 +606,11 @@ class ReadlineShell(BaseShell, cmd.Cmd):
                     pass
 
     def cmdloop(self, intro=None):
-        while not builtins.__xonsh__.exit:
+        while not XSH.exit:
             try:
                 self._cmdloop(intro=intro)
             except (KeyboardInterrupt, SystemExit):
-                print()  # Gives a newline
+                print(file=self.stdout)  # Gives a newline
                 fix_readline_state_after_ctrl_c()
                 self.reset_buffer()
                 intro = None
@@ -607,7 +631,7 @@ class ReadlineShell(BaseShell, cmd.Cmd):
                     print_exception()
                     self.mlprompt = "<multiline prompt error> "
             return self.mlprompt
-        env = builtins.__xonsh__.env  # pylint: disable=no-member
+        env = XSH.env  # pylint: disable=no-member
         p = env.get("PROMPT")
         try:
             p = self.prompt_formatter(p)
@@ -624,7 +648,7 @@ class ReadlineShell(BaseShell, cmd.Cmd):
         codes.
         """
         hide = hide if self._force_hide is None else self._force_hide
-        style = builtins.__xonsh__.env.get("XONSH_COLOR_STYLE")
+        style = XSH.env.get("XONSH_COLOR_STYLE")
         return ansi_partial_color_format(string, hide=hide, style=style)
 
     def print_color(self, string, hide=False, **kwargs):
@@ -632,7 +656,7 @@ class ReadlineShell(BaseShell, cmd.Cmd):
             s = self.format_color(string, hide=hide)
         else:
             # assume this is a list of (Token, str) tuples and format it
-            env = builtins.__xonsh__.env
+            env = XSH.env
             style_overrides_env = env.get("XONSH_STYLE_OVERRIDES", {})
             self.styler.style_name = env.get("XONSH_COLOR_STYLE")
             self.styler.override(style_overrides_env)
@@ -647,7 +671,7 @@ class ReadlineShell(BaseShell, cmd.Cmd):
 
     def color_style(self):
         """Returns the current color map."""
-        style = style = builtins.__xonsh__.env.get("XONSH_COLOR_STYLE")
+        style = XSH.env.get("XONSH_COLOR_STYLE")
         return ansi_color_style(style=style)
 
     def restore_tty_sanity(self):
@@ -657,7 +681,7 @@ class ReadlineShell(BaseShell, cmd.Cmd):
         """
         if not ON_POSIX:
             return
-        stty, _ = builtins.__xonsh__.commands_cache.lazyget("stty", (None, None))
+        stty, _ = XSH.commands_cache.lazyget("stty", (None, None))
         if stty is None:
             return
         # If available, we should just call the stty utility. This call should
@@ -687,7 +711,7 @@ class ReadlineHistoryAdder(threading.Thread):
             import readline
         except ImportError:
             return
-        hist = builtins.__xonsh__.history
+        hist = XSH.history
         if hist is None:
             return
         i = 1
