@@ -1,5 +1,4 @@
 """Xonsh color styling tools that simulate pygments, when it is unavailable."""
-import builtins
 from collections import defaultdict
 
 from xonsh.platform import HAS_PYGMENTS
@@ -64,9 +63,11 @@ def partial_color_tokenize(template):
     of tuples mapping the token to the string which has that color.
     These sub-strings maybe templates themselves.
     """
-    if HAS_PYGMENTS and builtins.__xonsh__.shell is not None:
-        styles = __xonsh__.shell.shell.styler.styles
-    elif builtins.__xonsh__.shell is not None:
+    from xonsh.built_ins import XSH
+
+    if HAS_PYGMENTS and XSH.shell is not None:
+        styles = XSH.shell.shell.styler.styles
+    elif XSH.shell is not None:
         styles = DEFAULT_STYLE_DICT
     else:
         styles = None
