@@ -17,13 +17,11 @@ Implementations:
 * indent()
 
 """
-import builtins
 import collections
 import collections.abc as cabc
 import contextlib
 import ctypes
 import datetime
-from distutils.version import LooseVersion
 import functools
 import glob
 import itertools
@@ -39,13 +37,13 @@ import operator
 import ast
 import string
 import typing as tp
+import shlex
 
 # adding imports from further xonsh modules is discouraged to avoid circular
 # dependencies
 from xonsh import __version__
 from xonsh.lazyasd import LazyObject, LazyDict, lazyobject
 from xonsh.platform import (
-    scandir,
     DEFAULT_ENCODING,
     ON_LINUX,
     ON_WINDOWS,
@@ -64,6 +62,13 @@ def is_superuser():
     return rtn
 
 
+@lazyobject
+def xsh():
+    from xonsh.built_ins import XSH
+
+    return XSH
+
+
 class XonshError(Exception):
     pass
 
@@ -100,8 +105,7 @@ class XonshCalledProcessError(XonshError, subprocess.CalledProcessError):
 def expand_path(s, expand_user=True):
     """Takes a string path and expands ~ to home if expand_user is set
     and environment vars if EXPAND_ENV_VARS is set."""
-    session = getattr(builtins, "__xonsh__", None)
-    env = os_environ if session is None else getattr(session, "env", os_environ)
+    env = xsh.env or os_environ
     if env.get("EXPAND_ENV_VARS", False):
         s = expandvars(s)
     if expand_user:
@@ -122,8 +126,7 @@ def _expandpath(path):
     """Performs environment variable / user expansion on a given path
     if EXPAND_ENV_VARS is set.
     """
-    session = getattr(builtins, "__xonsh__", None)
-    env = os_environ if session is None else getattr(session, "env", os_environ)
+    env = xsh.env or os_environ
     expand_user = env.get("EXPAND_ENV_VARS", False)
     return expand_path(path, expand_user=expand_user)
 
@@ -140,8 +143,7 @@ def decode_bytes(b):
     """Tries to decode the bytes using XONSH_ENCODING if available,
     otherwise using sys.getdefaultencoding().
     """
-    session = getattr(builtins, "__xonsh__", None)
-    env = os_environ if session is None else getattr(session, "env", os_environ)
+    env = xsh.env or os_environ
     enc = env.get("XONSH_ENCODING") or DEFAULT_ENCODING
     err = env.get("XONSH_ENCODING_ERRORS") or "strict"
     return b.decode(encoding=enc, errors=err)
@@ -233,7 +235,7 @@ class EnvPath(cabc.MutableSequence):
         return all(map(operator.eq, self, other))
 
     def _repr_pretty_(self, p, cycle):
-        """ Pretty print path list """
+        """Pretty print path list"""
         if cycle:
             p.text("EnvPath(...)")
         else:
@@ -278,10 +280,12 @@ class EnvPath(cabc.MutableSequence):
         None
 
         """
+        data = str(expand_path(data))
         if data not in self._l:
             self._l.insert(0 if front else len(self._l), data)
         elif replace:
-            self._l.remove(data)
+            # https://stackoverflow.com/a/25251306/1621381
+            self._l = list(filter(lambda x: x != data, self._l))
             self._l.insert(0 if front else len(self._l), data)
 
 
@@ -333,7 +337,7 @@ def balanced_parens(line, mincol=0, maxcol=None, lexer=None):
     """Determines if parentheses are balanced in an expression."""
     line = line[mincol:maxcol]
     if lexer is None:
-        lexer = builtins.__xonsh__.execer.parser.lexer
+        lexer = xsh.execer.parser.lexer
     if "(" not in line and ")" not in line:
         return True
     cnt = 0
@@ -348,6 +352,15 @@ def balanced_parens(line, mincol=0, maxcol=None, lexer=None):
     return cnt == 0
 
 
+def ends_with_colon_token(line, lexer=None):
+    """Determines whether a line ends with a colon token, ignoring comments."""
+    if lexer is None:
+        lexer = xsh.execer.parser.lexer
+    lexer.input(line)
+    toks = list(lexer)
+    return len(toks) > 0 and toks[-1].type == "COLON"
+
+
 def find_next_break(line, mincol=0, lexer=None):
     """Returns the column number of the next logical break in subproc mode.
     This function may be useful in finding the maxcol argument of
@@ -356,7 +369,7 @@ def find_next_break(line, mincol=0, lexer=None):
     if mincol >= 1:
         line = line[mincol:]
     if lexer is None:
-        lexer = builtins.__xonsh__.execer.parser.lexer
+        lexer = xsh.execer.parser.lexer
     if RE_END_TOKS.search(line) is None:
         return None
     maxcol = None
@@ -394,7 +407,7 @@ def subproc_toks(
     normal parentheses. Greedy is False by default.
     """
     if lexer is None:
-        lexer = builtins.__xonsh__.execer.parser.lexer
+        lexer = xsh.execer.parser.lexer
     if maxcol is None:
         maxcol = len(line) + 1
     lexer.reset()
@@ -529,11 +542,7 @@ def get_line_continuation():
     mode on Windows the backslash must be preceded by a space. This is because
     paths on Windows may end in a backslash.
     """
-    if (
-        ON_WINDOWS
-        and hasattr(builtins.__xonsh__, "env")
-        and builtins.__xonsh__.env.get("XONSH_INTERACTIVE", False)
-    ):
+    if ON_WINDOWS and hasattr(xsh, "env") and xsh.env.get("XONSH_INTERACTIVE", False):
         return " \\"
     else:
         return "\\"
@@ -618,7 +627,7 @@ def subexpr_before_unbalanced(expr, ltok, rtok):
     subexpr, _, post = expr.rpartition(ltok)
     nrtoks_in_post = post.count(rtok)
     while nrtoks_in_post != 0:
-        for i in range(nrtoks_in_post):
+        for _ in range(nrtoks_in_post):
             subexpr, _, post = subexpr.rpartition(ltok)
         nrtoks_in_post = post.count(rtok)
     _, _, subexpr = subexpr.rpartition(rtok)
@@ -659,7 +668,7 @@ def safe_hasattr(obj, attr):
     try:
         getattr(obj, attr)
         return True
-    except Exception:  # pylint:disable=bare-except
+    except Exception:
         return False
 
 
@@ -704,7 +713,7 @@ def get_sep():
     """Returns the appropriate filepath separator char depending on OS and
     xonsh options set
     """
-    if ON_WINDOWS and builtins.__xonsh__.env.get("FORCE_POSIX_PATHS"):
+    if ON_WINDOWS and xsh.env.get("FORCE_POSIX_PATHS"):
         return os.altsep
     else:
         return os.sep
@@ -771,7 +780,7 @@ def _yield_accessible_unix_file_names(path):
     """yield file names of executable files in path."""
     if not os.path.exists(path):
         return
-    for file_ in scandir(path):
+    for file_ in os.scandir(path):
         try:
             if file_.is_file() and os.access(file_.path, os.X_OK):
                 yield file_.name
@@ -790,9 +799,9 @@ def _executables_in_posix(path):
 def _executables_in_windows(path):
     if not os.path.isdir(path):
         return
-    extensions = builtins.__xonsh__.env["PATHEXT"]
+    extensions = xsh.env["PATHEXT"]
     try:
-        for x in scandir(path):
+        for x in os.scandir(path):
             try:
                 is_file = x.is_file()
             except OSError:
@@ -812,7 +821,7 @@ def _executables_in_windows(path):
 
 
 def executables_in(path):
-    """Returns a generator of files in path that the user could execute. """
+    """Returns a generator of files in path that the user could execute."""
     if ON_WINDOWS:
         func = _executables_in_windows
     else:
@@ -830,7 +839,7 @@ def debian_command_not_found(cmd):
     if not ON_LINUX:
         return ""
 
-    cnf = builtins.__xonsh__.commands_cache.lazyget(
+    cnf = xsh.commands_cache.lazyget(
         "command-not-found", ("/usr/lib/command-not-found",)
     )[0]
 
@@ -839,7 +848,7 @@ def debian_command_not_found(cmd):
 
     c = "{0} {1}; exit 0"
     s = subprocess.check_output(
-        c.format(cnf, cmd),
+        c.format(cnf, shlex.quote(cmd)),
         universal_newlines=True,
         stderr=subprocess.STDOUT,
         shell=True,
@@ -875,7 +884,8 @@ def command_not_found(cmd, env):
     return rtn
 
 
-def suggest_commands(cmd, env, aliases):
+@functools.lru_cache()
+def suggest_commands(cmd, env):
     """Suggests alternative commands given an environment and aliases."""
     if not env.get("SUGGEST_COMMANDS"):
         return ""
@@ -886,12 +896,12 @@ def suggest_commands(cmd, env, aliases):
     cmd = cmd.lower()
     suggested = {}
 
-    for alias in builtins.aliases:
+    for alias in xsh.aliases:
         if alias not in suggested:
             if levenshtein(alias.lower(), cmd, thresh) < thresh:
                 suggested[alias] = "Alias"
 
-    for _cmd in builtins.__xonsh__.commands_cache.all_commands:
+    for _cmd in xsh.commands_cache.all_commands:
         if _cmd not in suggested:
             if levenshtein(_cmd.lower(), cmd, thresh) < thresh:
                 suggested[_cmd] = "Command ({0})".format(_cmd)
@@ -921,7 +931,7 @@ def suggest_commands(cmd, env, aliases):
 
 def _get_manual_env_var(name, default=None):
     """Returns if the given variable is manually set as well as it's value."""
-    env = getattr(builtins.__xonsh__, "env", None)
+    env = getattr(xsh, "env", None)
     if env is None:
         env = os_environ
         manually_set = name in env
@@ -1452,8 +1462,8 @@ def get_portions(it, slices):
 
     Parameters
     ----------
-    it: iterable
-    slices: a slice or a list of slice objects
+    it : iterable
+    slices : a slice or a list of slice objects
     """
     if is_slice(slices):
         slices = [slices]
@@ -1866,13 +1876,13 @@ def format_color(string, **kwargs):
     shell instances method of the same name. The results of this function should
     be directly usable by print_color().
     """
-    if hasattr(builtins.__xonsh__.shell, "shell"):
-        return builtins.__xonsh__.shell.shell.format_color(string, **kwargs)
+    if hasattr(xsh.shell, "shell"):
+        return xsh.shell.shell.format_color(string, **kwargs)
     else:
         # fallback for ANSI if shell is not yet initialized
         from xonsh.ansi_colors import ansi_partial_color_format
 
-        style = builtins.__xonsh__.env.get("XONSH_COLOR_STYLE")
+        style = xsh.env.get("XONSH_COLOR_STYLE")
         return ansi_partial_color_format(string, style=style)
 
 
@@ -1881,8 +1891,8 @@ def print_color(string, **kwargs):
     method of the same name. Colors will be formatted if they have not already
     been.
     """
-    if hasattr(builtins.__xonsh__.shell, "shell"):
-        builtins.__xonsh__.shell.shell.print_color(string, **kwargs)
+    if hasattr(xsh.shell, "shell"):
+        xsh.shell.shell.print_color(string, **kwargs)
     else:
         # fallback for ANSI if shell is not yet initialized
         print(format_color(string, **kwargs))
@@ -1890,12 +1900,12 @@ def print_color(string, **kwargs):
 
 def color_style_names():
     """Returns an iterable of all available style names."""
-    return builtins.__xonsh__.shell.shell.color_style_names()
+    return xsh.shell.shell.color_style_names()
 
 
 def color_style():
     """Returns the current color map."""
-    return builtins.__xonsh__.shell.shell.color_style()
+    return xsh.shell.shell.color_style()
 
 
 def register_custom_style(
@@ -1937,10 +1947,10 @@ def register_custom_style(
 
 
 def _token_attr_from_stylemap(stylemap):
-    """yields tokens attr, and index from a stylemap """
+    """yields tokens attr, and index from a stylemap"""
     import prompt_toolkit as ptk
 
-    if builtins.__xonsh__.shell.shell_type == "prompt_toolkit1":
+    if xsh.shell.shell_type == "prompt_toolkit1":
         style = ptk.styles.style_from_dict(stylemap)
         for token in stylemap:
             yield token, style.token_to_attrs[token]
@@ -1954,8 +1964,8 @@ def _token_attr_from_stylemap(stylemap):
 
 
 def _get_color_lookup_table():
-    """Returns the prompt_toolkit win32 ColorLookupTable """
-    if builtins.__xonsh__.shell.shell_type == "prompt_toolkit1":
+    """Returns the prompt_toolkit win32 ColorLookupTable"""
+    if xsh.shell.shell_type == "prompt_toolkit1":
         from prompt_toolkit.terminal.win32_output import ColorLookupTable
     else:
         from prompt_toolkit.output.win32 import ColorLookupTable
@@ -1963,7 +1973,7 @@ def _get_color_lookup_table():
 
 
 def _get_color_indexes(style_map):
-    """Generates the color and windows color index for a style """
+    """Generates the color and windows color index for a style"""
     table = _get_color_lookup_table()
     for token, attr in _token_attr_from_stylemap(style_map):
         if attr.color:
@@ -2039,7 +2049,7 @@ WIN10_COLOR_MAP = LazyObject(_win10_color_map, globals(), "WIN10_COLOR_MAP")
 
 
 def _win_bold_color_map():
-    """ Map dark ansi colors to lighter version. """
+    """Map dark ansi colors to lighter version."""
     return {
         "ansiblack": "ansibrightblack",
         "ansiblue": "ansibrightblue",
@@ -2060,8 +2070,8 @@ def hardcode_colors_for_win10(style_map):
     in conhost.exe
     """
     modified_style = {}
-    if not builtins.__xonsh__.env["PROMPT_TOOLKIT_COLOR_DEPTH"]:
-        builtins.__xonsh__.env["PROMPT_TOOLKIT_COLOR_DEPTH"] = "DEPTH_24_BIT"
+    if not xsh.env["PROMPT_TOOLKIT_COLOR_DEPTH"]:
+        xsh.env["PROMPT_TOOLKIT_COLOR_DEPTH"] = "DEPTH_24_BIT"
     # Replace all ansi colors with hardcoded colors to avoid unreadable defaults
     # in conhost.exe
     for token, style_str in style_map.items():
@@ -2108,7 +2118,7 @@ def intensify_colors_for_cmd_exe(style_map):
         6: "ansibrightyellow",  # subst yellow with bright yellow
         9: "ansicyan",  # subst intense blue with dark cyan (more readable)
     }
-    if builtins.__xonsh__.shell.shell_type == "prompt_toolkit1":
+    if xsh.shell.shell_type == "prompt_toolkit1":
         replace_colors = ansicolors_to_ptk1_names(replace_colors)
     for token, idx, _ in _get_color_indexes(style_map):
         if idx in replace_colors:
@@ -2122,11 +2132,11 @@ def intensify_colors_on_win_setter(enable):
     """
     enable = to_bool(enable)
     if (
-        hasattr(builtins.__xonsh__, "shell")
-        and builtins.__xonsh__.shell is not None
-        and hasattr(builtins.__xonsh__.shell.shell.styler, "style_name")
+        hasattr(xsh, "shell")
+        and xsh.shell is not None
+        and hasattr(xsh.shell.shell.styler, "style_name")
     ):
-        delattr(builtins.__xonsh__.shell.shell.styler, "style_name")
+        delattr(xsh.shell.shell.styler, "style_name")
     return enable
 
 
@@ -2136,9 +2146,9 @@ def format_std_prepost(template, env=None):
     """
     if not template:
         return ""
-    env = builtins.__xonsh__.env if env is None else env
+    env = xsh.env if env is None else env
     invis = "\001\002"
-    if builtins.__xonsh__.shell is None:
+    if xsh.shell is None:
         # shell hasn't fully started up (probably still in xonshrc)
         from xonsh.prompt.base import PromptFormatter
         from xonsh.ansi_colors import ansi_partial_color_format
@@ -2149,7 +2159,7 @@ def format_std_prepost(template, env=None):
         s = ansi_partial_color_format(invis + s + invis, hide=False, style=style)
     else:
         # shell has fully started. do the normal thing
-        shell = builtins.__xonsh__.shell.shell
+        shell = xsh.shell.shell
         try:
             s = shell.prompt_formatter(template)
         except Exception:
@@ -2324,7 +2334,7 @@ def POSIX_ENVVAR_REGEX():
 def expandvars(path):
     """Expand shell variables of the forms $var, ${var} and %var%.
     Unknown variables are left unchanged."""
-    env = builtins.__xonsh__.env
+    env = xsh.env
     if isinstance(path, bytes):
         path = path.decode(
             encoding=env.get("XONSH_ENCODING"), errors=env.get("XONSH_ENCODING_ERRORS")
@@ -2373,7 +2383,7 @@ def normabspath(p):
 
 
 def expanduser_abs_path(inp):
-    """ Provides user expanded absolute path """
+    """Provides user expanded absolute path"""
     return os.path.abspath(expanduser(inp))
 
 
@@ -2445,11 +2455,11 @@ def _dotglobstr(s):
 
 
 def _iglobpath(s, ignore_case=False, sort_result=None, include_dotfiles=None):
-    s = builtins.__xonsh__.expand_path(s)
+    s = xsh.expand_path(s)
     if sort_result is None:
-        sort_result = builtins.__xonsh__.env.get("GLOB_SORTED")
+        sort_result = xsh.env.get("GLOB_SORTED")
     if include_dotfiles is None:
-        include_dotfiles = builtins.__xonsh__.env.get("DOTGLOB")
+        include_dotfiles = xsh.env.get("DOTGLOB")
     if ignore_case:
         s = expand_case_matching(s)
     if "**" in s and "**/*" not in s:
@@ -2491,7 +2501,7 @@ def ensure_timestamp(t, datetime_format=None):
     except (ValueError, TypeError):
         pass
     if datetime_format is None:
-        datetime_format = builtins.__xonsh__.env["XONSH_DATETIME_FORMAT"]
+        datetime_format = xsh.env["XONSH_DATETIME_FORMAT"]
     if isinstance(t, datetime.datetime):
         t = t.timestamp()
     else:
@@ -2501,7 +2511,7 @@ def ensure_timestamp(t, datetime_format=None):
 
 def format_datetime(dt):
     """Format datetime object to string base on $XONSH_DATETIME_FORMAT Env."""
-    format_ = builtins.__xonsh__.env["XONSH_DATETIME_FORMAT"]
+    format_ = xsh.env["XONSH_DATETIME_FORMAT"]
     return dt.strftime(format_)
 
 
@@ -2640,9 +2650,48 @@ def _deprecated_message_suffix(deprecated_in, removed_in):
 
 
 def _deprecated_error_on_expiration(name, removed_in):
+    from distutils.version import LooseVersion
+
     if not removed_in:
         return
     elif LooseVersion(__version__) >= LooseVersion(removed_in):
         raise AssertionError(
             "{} has passed its version {} expiry date!".format(name, removed_in)
         )
+
+
+def to_repr_pretty_(inst, p, cycle):
+    name = "{0}.{1}".format(inst.__class__.__module__, inst.__class__.__name__)
+    with p.group(0, name + "(", ")"):
+        if cycle:
+            p.text("...")
+        elif len(inst):
+            p.break_()
+            p.pretty(dict(inst))
+
+
+class XAttr:
+    """hold attribute and value"""
+
+    __slots__ = ("name", "value")
+
+    def __init__(self, val) -> None:
+        self.value = val
+
+    def __set_name__(self, owner, name) -> None:
+        self.name: str = name
+
+    def __get__(self, instance, owner) -> "XAttr":
+        return self
+
+    def __str__(self) -> str:
+        return f"<{self.name}={self.value}>"
+
+
+class NamedConstantMeta(type):
+    """utility class to hold list of values as class-attributes"""
+
+    def __iter__(cls) -> tp.Iterator[XAttr]:
+        for attr in vars(cls):
+            if not attr.startswith("__"):
+                yield getattr(cls, attr)
