@@ -6,12 +6,12 @@ import ast
 import json
 import pprint
 import fnmatch
-import builtins
 import textwrap
 import collections.abc as cabc
 import typing as tp
 
 from xonsh.tools import to_bool, to_bool_or_break, backup_file, print_color
+from xonsh.built_ins import XSH
 from xonsh.jsonutils import serialize_xonsh_json
 
 
@@ -709,13 +709,13 @@ class PromptVisitor(StateVisitor):
             Tree of nodes to start visitor with.
         state : dict, optional
             Initial state to begin with.
-        kwargs : optional
+        **kwargs : optional
             Options that are passed through to the prompt via the shell's
             singleline() method. See BaseShell for mor details.
         """
         super().__init__(tree=tree, state=state)
-        self.env = builtins.__xonsh__.env
-        self.shell = builtins.__xonsh__.shell.shell
+        self.env = XSH.env
+        self.shell = XSH.shell.shell
         self.shell_kwargs = kwargs
 
     def visit_wizard(self, node):
