@@ -2,22 +2,24 @@
 import os
 import re
 import ast
+import sys
 import json
 import shutil
 import random
 import pprint
+import tempfile
 import textwrap
-import builtins
-import argparse
-import functools
 import itertools
 import contextlib
 import collections
+import typing as tp
 
 from xonsh.ply import ply
 
 import xonsh.wizard as wiz
 from xonsh import __version__ as XONSH_VERSION
+from xonsh.built_ins import XSH
+from xonsh.cli_utils import ArgParserAlias, Annotated, Arg, add_args
 from xonsh.prompt.base import is_template_string
 from xonsh.platform import (
     is_readline_available,
@@ -27,6 +29,8 @@ from xonsh.platform import (
     ON_POSIX,
     ON_LINUX,
     linux_distro,
+    ON_WSL,
+    ON_WSL1,
     ON_DARWIN,
     ON_WINDOWS,
     ON_CYGWIN,
@@ -44,13 +48,14 @@ from xonsh.tools import (
     color_style,
 )
 from xonsh.foreign_shells import CANON_SHELL_NAMES
-from xonsh.xontribs import xontrib_metadata, find_xontrib
+from xonsh.xontribs import find_xontrib, xontribs_loaded
+from xonsh.xontribs_meta import get_xontribs, Xontrib
 from xonsh.lazyasd import lazyobject
 
 HR = "'`-.,_,.-*'`-.,_,.-*'`-.,_,.-*'`-.,_,.-*'`-.,_,.-*'`-.,_,.-*'`-.,_,.-*'"
 WIZARD_HEAD = """
-          {{BOLD_WHITE}}Welcome to the xonsh configuration wizard!{{NO_COLOR}}
-          {{YELLOW}}------------------------------------------{{NO_COLOR}}
+          {{BOLD_WHITE}}Welcome to the xonsh configuration wizard!{{RESET}}
+          {{YELLOW}}------------------------------------------{{RESET}}
 This will present a guided tour through setting up the xonsh static
 config file. Xonsh will automatically ask you if you want to run this
 wizard if the configuration file does not exist. However, you can
@@ -75,10 +80,10 @@ For the configuration to take effect, you will need to restart xonsh.
 WIZARD_FS = """
 {hr}
 
-                      {{BOLD_WHITE}}Foreign Shell Setup{{NO_COLOR}}
-                      {{YELLOW}}-------------------{{NO_COLOR}}
-The xonsh shell has the ability to interface with foreign shells such
-as Bash, or zsh (fish not yet implemented).
+                      {{BOLD_WHITE}}Foreign Shell Setup{{RESET}}
+                      {{YELLOW}}-------------------{{RESET}}
+The xonsh shell has the ability to interface with Bash or zsh
+via the foreign shell interface.
 
 For configuration, this means that xonsh can load the environment,
 aliases, and functions specified in the config files of these shells.
@@ -92,8 +97,8 @@ makes it easier to transition to and from xonsh.
 WIZARD_ENV = """
 {hr}
 
-                  {{BOLD_WHITE}}Environment Variable Setup{{NO_COLOR}}
-                  {{YELLOW}}--------------------------{{NO_COLOR}}
+                  {{BOLD_WHITE}}Environment Variable Setup{{RESET}}
+                  {{YELLOW}}--------------------------{{RESET}}
 The xonsh shell also allows you to setup environment variables from
 the static configuration file. Any variables set in this way are
 superseded by the definitions in the xonshrc or on the command line.
@@ -104,7 +109,7 @@ The following lists the environment variable name, its documentation,
 the default value, and the current value. The default and current
 values are presented as pretty repr strings of their Python types.
 
-{{BOLD_GREEN}}Note:{{NO_COLOR}} Simply hitting enter for any environment variable
+{{BOLD_GREEN}}Note:{{RESET}} Simply hitting enter for any environment variable
 will accept the default value for that entry.
 """.format(
     hr=HR
@@ -115,10 +120,10 @@ WIZARD_ENV_QUESTION = "Would you like to set env vars now, " + wiz.YN
 WIZARD_XONTRIB = """
 {hr}
 
-                           {{BOLD_WHITE}}Xontribs{{NO_COLOR}}
-                           {{YELLOW}}--------{{NO_COLOR}}
+                           {{BOLD_WHITE}}Xontribs{{RESET}}
+                           {{YELLOW}}--------{{RESET}}
 No shell is complete without extensions, and xonsh is no exception. Xonsh
-extensions are called {{BOLD_GREEN}}xontribs{{NO_COLOR}}, or xonsh contributions.
+extensions are called {{BOLD_GREEN}}xontribs{{RESET}}, or xonsh contributions.
 Xontribs are dynamically loadable, either by importing them directly or by
 using the 'xontrib' command. However, you can also configure xonsh to load
 xontribs automatically on startup prior to loading the run control files.
@@ -136,10 +141,12 @@ WIZARD_TAIL = """
 Thanks for using the xonsh configuration wizard!"""
 
 
-_XONFIG_SOURCE_FOREIGN_SHELL_COMMAND = collections.defaultdict(
+_XONFIG_SOURCE_FOREIGN_SHELL_COMMAND: tp.Dict[str, str] = collections.defaultdict(
     lambda: "source-foreign", bash="source-bash", cmd="source-cmd", zsh="source-zsh"
 )
 
+XONSH_JUPYTER_KERNEL = "xonsh"
+
 
 def _dump_xonfig_foreign_shell(path, value):
     shell = value["shell"]
@@ -183,8 +190,8 @@ def _dump_xonfig_foreign_shell(path, value):
 
 def _dump_xonfig_env(path, value):
     name = os.path.basename(path.rstrip("/"))
-    ensurer = builtins.__xonsh__.env.get_ensurer(name)
-    dval = str(value) if ensurer.detype is None else ensurer.detype(value)
+    detyper = XSH.env.get_detyper(name)
+    dval = str(value) if detyper is None else detyper(value)
     dval = str(value) if dval is None else dval
     return "${name} = {val!r}".format(name=name, val=dval)
 
@@ -273,31 +280,31 @@ def _wrap_paragraphs(text, width=70, **kwargs):
 
 
 ENVVAR_MESSAGE = """
-{{BOLD_CYAN}}${name}{{NO_COLOR}}
+{{BOLD_CYAN}}${name}{{RESET}}
 {docstr}
-{{RED}}default value:{{NO_COLOR}} {default}
-{{RED}}current value:{{NO_COLOR}} {current}"""
+{{RED}}default value:{{RESET}} {default}
+{{RED}}current value:{{RESET}} {current}"""
 
-ENVVAR_PROMPT = "{BOLD_GREEN}>>>{NO_COLOR} "
+ENVVAR_PROMPT = "{BOLD_GREEN}>>>{RESET} "
 
 
 def make_exit_message():
     """Creates a message for how to exit the wizard."""
-    shell_type = builtins.__xonsh__.shell.shell_type
+    shell_type = XSH.shell.shell_type
     keyseq = "Ctrl-D" if shell_type == "readline" else "Ctrl-C"
     msg = "To exit the wizard at any time, press {BOLD_UNDERLINE_CYAN}"
-    msg += keyseq + "{NO_COLOR}.\n"
+    msg += keyseq + "{RESET}.\n"
     m = wiz.Message(message=msg)
     return m
 
 
 def make_envvar(name):
     """Makes a StoreNonEmpty node for an environment variable."""
-    env = builtins.__xonsh__.env
+    env = XSH.env
     vd = env.get_docs(name)
-    if not vd.configurable:
+    if not vd.is_configurable:
         return
-    default = vd.default
+    default = vd.doc_default
     if "\n" in default:
         default = "\n" + _wrap_paragraphs(default, width=69)
     curr = env.get(name)
@@ -310,18 +317,18 @@ def make_envvar(name):
         name=name,
         default=default,
         current=curr,
-        docstr=_wrap_paragraphs(vd.docstr, width=69),
+        docstr=_wrap_paragraphs(vd.doc, width=69),
     )
     mnode = wiz.Message(message=msg)
-    ens = env.get_ensurer(name)
+    converter = env.get_converter(name)
     path = "/env/" + name
     pnode = wiz.StoreNonEmpty(
         ENVVAR_PROMPT,
-        converter=ens.convert,
+        converter=converter,
         show_conversion=True,
         path=path,
         retry=True,
-        store_raw=vd.store_as_str,
+        store_raw=vd.can_store_as_str,
     )
     return mnode, pnode
 
@@ -339,11 +346,11 @@ def _make_flat_wiz(kidfunc, *args):
 
 def make_env_wiz():
     """Makes an environment variable wizard."""
-    w = _make_flat_wiz(make_envvar, sorted(builtins.__xonsh__.env._docs.keys()))
+    w = _make_flat_wiz(make_envvar, sorted(XSH.env.keys()))
     return w
 
 
-XONTRIB_PROMPT = "{BOLD_GREEN}Add this xontrib{NO_COLOR}, " + wiz.YN
+XONTRIB_PROMPT = "{BOLD_GREEN}Add this xontrib{RESET}, " + wiz.YN
 
 
 def _xontrib_path(visitor=None, node=None, val=None):
@@ -351,25 +358,24 @@ def _xontrib_path(visitor=None, node=None, val=None):
     return ("xontribs", len(visitor.state.get("xontribs", ())))
 
 
-def make_xontrib(xontrib, package):
+def make_xontrib(xon_item: tp.Tuple[str, Xontrib]):
     """Makes a message and StoreNonEmpty node for a xontrib."""
-    name = xontrib.get("name", "<unknown-xontrib-name>")
-    msg = "\n{BOLD_CYAN}" + name + "{NO_COLOR}\n"
-    if "url" in xontrib:
-        msg += "{RED}url:{NO_COLOR} " + xontrib["url"] + "\n"
-    if "package" in xontrib:
-        msg += "{RED}package:{NO_COLOR} " + xontrib["package"] + "\n"
-    if "url" in package:
-        if "url" in xontrib and package["url"] != xontrib["url"]:
-            msg += "{RED}package-url:{NO_COLOR} " + package["url"] + "\n"
-    if "license" in package:
-        msg += "{RED}license:{NO_COLOR} " + package["license"] + "\n"
-    msg += "{PURPLE}installed?{NO_COLOR} "
+    name, xontrib = xon_item
+    name = name or "<unknown-xontrib-name>"
+    msg = "\n{BOLD_CYAN}" + name + "{RESET}\n"
+    if xontrib.url:
+        msg += "{RED}url:{RESET} " + xontrib.url + "\n"
+    if xontrib.package:
+        pkg = xontrib.package
+        msg += "{RED}package:{RESET} " + pkg.name + "\n"
+        if pkg.url:
+            if xontrib.url and pkg.url != xontrib.url:
+                msg += "{RED}package-url:{RESET} " + pkg.url + "\n"
+        if pkg.license:
+            msg += "{RED}license:{RESET} " + pkg.license + "\n"
+    msg += "{PURPLE}installed?{RESET} "
     msg += ("no" if find_xontrib(name) is None else "yes") + "\n"
-    desc = xontrib.get("description", "")
-    if not isinstance(desc, str):
-        desc = "".join(desc)
-    msg += _wrap_paragraphs(desc, width=69)
+    msg += _wrap_paragraphs(xontrib.description, width=69)
     if msg.endswith("\n"):
         msg = msg[:-1]
     mnode = wiz.Message(message=msg)
@@ -380,10 +386,7 @@ def make_xontrib(xontrib, package):
 
 def make_xontribs_wiz():
     """Makes a xontrib wizard."""
-    md = xontrib_metadata()
-    pkgs = [md["packages"].get(d.get("package", None), {}) for d in md["xontribs"]]
-    w = _make_flat_wiz(make_xontrib, md["xontribs"], pkgs)
-    return w
+    return _make_flat_wiz(make_xontrib, get_xontribs().items())
 
 
 def make_xonfig_wizard(default_file=None, confirm=False, no_wizard_file=None):
@@ -441,14 +444,25 @@ def make_xonfig_wizard(default_file=None, confirm=False, no_wizard_file=None):
     return w
 
 
-def _wizard(ns):
-    env = builtins.__xonsh__.env
-    shell = builtins.__xonsh__.shell.shell
-    fname = env.get("XONSHRC")[-1] if ns.file is None else ns.file
+def _wizard(
+    rcfile=None,
+    confirm=False,
+):
+    """Launch configurator in terminal
+
+    Parameters
+    -------
+    rcfile : -f, --file
+        config file location, default=$XONSHRC
+    confirm : -c, --confirm
+        confirm that the wizard should be run.
+    """
+    env = XSH.env
+    shell = XSH.shell.shell
+    xonshrcs = env.get("XONSHRC", [])
+    fname = xonshrcs[-1] if xonshrcs and rcfile is None else rcfile
     no_wiz = os.path.join(env.get("XONSH_CONFIG_DIR"), "no-wizard")
-    w = make_xonfig_wizard(
-        default_file=fname, confirm=ns.confirm, no_wizard_file=no_wiz
-    )
+    w = make_xonfig_wizard(default_file=fname, confirm=confirm, no_wizard_file=no_wiz)
     tempenv = {"PROMPT": "", "XONSH_STORE_STDOUT": False}
     pv = wiz.PromptVisitor(w, store_in_history=False, multiline=False)
 
@@ -473,12 +487,22 @@ def _xonfig_format_human(data):
     wcol1 = wcol2 = 0
     for key, val in data:
         wcol1 = max(wcol1, len(key))
-        wcol2 = max(wcol2, len(str(val)))
+        if isinstance(val, list):
+            for subval in val:
+                wcol2 = max(wcol2, len(str(subval)))
+        else:
+            wcol2 = max(wcol2, len(str(val)))
     hr = "+" + ("-" * (wcol1 + 2)) + "+" + ("-" * (wcol2 + 2)) + "+\n"
     row = "| {key!s:<{wcol1}} | {val!s:<{wcol2}} |\n"
     s = hr
     for key, val in data:
-        s += row.format(key=key, wcol1=wcol1, val=val, wcol2=wcol2)
+        if isinstance(val, list) and val:
+            for i, subval in enumerate(val):
+                s += row.format(
+                    key=f"{key} {i+1}", wcol1=wcol1, val=subval, wcol2=wcol2
+                )
+        else:
+            s += row.format(key=key, wcol1=wcol1, val=val, wcol2=wcol2)
     s += hr
     return s
 
@@ -489,9 +513,18 @@ def _xonfig_format_json(data):
     return s
 
 
-def _info(ns):
-    env = builtins.__xonsh__.env
-    data = [("xonsh", XONSH_VERSION)]
+def _info(
+    to_json=False,
+) -> str:
+    """Displays configuration information
+
+    Parameters
+    ----------
+    to_json : -j, --json
+        reports results as json
+    """
+    env = XSH.env
+    data: tp.List[tp.Any] = [("xonsh", XONSH_VERSION)]
     hash_, date_ = githash()
     if hash_:
         data.append(("Git SHA", hash_))
@@ -503,6 +536,7 @@ def _info(ns):
             ("have readline", is_readline_available()),
             ("prompt toolkit", ptk_version() or None),
             ("shell type", env.get("SHELL_TYPE")),
+            ("history backend", env.get("XONSH_HISTORY_BACKEND")),
             ("pygments", pygments_version()),
             ("on posix", bool(ON_POSIX)),
             ("on linux", bool(ON_LINUX)),
@@ -510,39 +544,62 @@ def _info(ns):
     )
     if ON_LINUX:
         data.append(("distro", linux_distro()))
+        data.append(("on wsl", bool(ON_WSL)))
+        if ON_WSL:
+            data.append(("wsl version", 1 if ON_WSL1 else 2))
     data.extend(
         [
-            ("on darwin", ON_DARWIN),
-            ("on windows", ON_WINDOWS),
-            ("on cygwin", ON_CYGWIN),
-            ("on msys2", ON_MSYS),
+            ("on darwin", bool(ON_DARWIN)),
+            ("on windows", bool(ON_WINDOWS)),
+            ("on cygwin", bool(ON_CYGWIN)),
+            ("on msys2", bool(ON_MSYS)),
             ("is superuser", is_superuser()),
             ("default encoding", DEFAULT_ENCODING),
             ("xonsh encoding", env.get("XONSH_ENCODING")),
             ("encoding errors", env.get("XONSH_ENCODING_ERRORS")),
         ]
     )
-    formatter = _xonfig_format_json if ns.json else _xonfig_format_human
+    jup_ksm = jup_kernel = None
+    try:
+        from jupyter_client.kernelspec import KernelSpecManager
+
+        jup_ksm = KernelSpecManager()
+        jup_kernel = jup_ksm.find_kernel_specs().get(XONSH_JUPYTER_KERNEL)
+    except Exception:
+        pass
+    data.extend([("on jupyter", jup_ksm is not None), ("jupyter kernel", jup_kernel)])
+
+    data.extend([("xontrib", xontribs_loaded())])
+    data.extend([("RC file", XSH.rc_files)])
+
+    formatter = _xonfig_format_json if to_json else _xonfig_format_human
     s = formatter(data)
     return s
 
 
-def _styles(ns):
-    env = builtins.__xonsh__.env
+def _styles(to_json=False, _stdout=None):
+    """Prints available xonsh color styles
+
+    Parameters
+    ----------
+    to_json: -j, --json
+        reports results as json
+    """
+    env = XSH.env
     curr = env.get("XONSH_COLOR_STYLE")
     styles = sorted(color_style_names())
-    if ns.json:
+    if to_json:
         s = json.dumps(styles, sort_keys=True, indent=1)
         print(s)
         return
     lines = []
     for style in styles:
         if style == curr:
-            lines.append("* {GREEN}" + style + "{NO_COLOR}")
+            lines.append("* {GREEN}" + style + "{RESET}")
         else:
             lines.append("  " + style)
     s = "\n".join(lines)
-    print_color(s)
+    print_color(s, file=_stdout)
 
 
 def _str_colors(cmap, cols):
@@ -554,7 +611,7 @@ def _str_colors(cmap, cols):
         line = ""
         for i, name in enumerate(group):
             buf = " " * (width - len(name))
-            line += "{" + name + "}" + name + "{NO_COLOR}" + buf
+            line += "{" + name + "}" + name + "{RESET}" + buf
             if (i + 1) % n == 0:
                 lines.append(line)
                 line = ""
@@ -566,7 +623,7 @@ def _str_colors(cmap, cols):
 def _tok_colors(cmap, cols):
     from xonsh.style_tools import Color
 
-    nc = Color.NO_COLOR
+    nc = Color.RESET
     names_toks = {}
     for t in cmap.keys():
         name = str(t)
@@ -589,87 +646,163 @@ def _tok_colors(cmap, cols):
     return toks
 
 
-def _colors(args):
-    columns, _ = shutil.get_terminal_size()
-    columns -= int(ON_WINDOWS)
-    style_stash = builtins.__xonsh__.env["XONSH_COLOR_STYLE"]
+def xonfig_color_completer(*_, **__):
+    yield from color_style_names()
 
-    if args.style is not None:
-        if args.style not in color_style_names():
-            print("Invalid style: {}".format(args.style))
+
+def _colors(
+    style: Annotated[str, Arg(nargs="?", completer=xonfig_color_completer)] = None
+):
+    """Preview color style
+
+    Parameters
+    ----------
+    style
+        name of the style to preview. If not given, current style name is used.
+    """
+    columns, _ = shutil.get_terminal_size()
+    columns -= int(bool(ON_WINDOWS))
+    style_stash = XSH.env["XONSH_COLOR_STYLE"]
+
+    if style is not None:
+        if style not in color_style_names():
+            print("Invalid style: {}".format(style))
             return
-        builtins.__xonsh__.env["XONSH_COLOR_STYLE"] = args.style
+        XSH.env["XONSH_COLOR_STYLE"] = style
 
     color_map = color_style()
+    if not color_map:
+        print("Empty color map - using non-interactive shell?")
+        return
     akey = next(iter(color_map))
     if isinstance(akey, str):
         s = _str_colors(color_map, columns)
     else:
         s = _tok_colors(color_map, columns)
     print_color(s)
-    builtins.__xonsh__.env["XONSH_COLOR_STYLE"] = style_stash
+    XSH.env["XONSH_COLOR_STYLE"] = style_stash
 
 
-def _tutorial(args):
+def _tutorial():
+    """Launch tutorial in browser."""
     import webbrowser
 
     webbrowser.open("http://xon.sh/tutorial.html")
 
 
-@functools.lru_cache(1)
-def _xonfig_create_parser():
-    p = argparse.ArgumentParser(
-        prog="xonfig", description="Manages xonsh configuration."
-    )
-    subp = p.add_subparsers(title="action", dest="action")
-    info = subp.add_parser(
-        "info", help=("displays configuration information, " "default action")
-    )
-    info.add_argument(
-        "--json", action="store_true", default=False, help="reports results as json"
-    )
-    wiz = subp.add_parser("wizard", help="displays configuration information")
-    wiz.add_argument(
-        "--file", default=None, help="config file location, default=$XONSHRC"
-    )
-    wiz.add_argument(
-        "--confirm",
-        action="store_true",
-        default=False,
-        help="confirm that the wizard should be run.",
-    )
-    sty = subp.add_parser("styles", help="prints available xonsh color styles")
-    sty.add_argument(
-        "--json", action="store_true", default=False, help="reports results as json"
-    )
-    colors = subp.add_parser("colors", help="preview color style")
-    colors.add_argument(
-        "style", nargs="?", default=None, help="style to preview, default: <current>"
-    )
-    subp.add_parser("tutorial", help="Launch tutorial in browser.")
-    return p
+def _web(
+    _args,
+    browser=True,
+):
+    """Launch configurator in browser.
+
+    Parameters
+    ----------
+    browser : --nb, --no-browser
+        don't open browser
+    """
+
+    import subprocess
+
+    subprocess.run([sys.executable, "-m", "xonsh.webconfig"] + _args[1:])
 
 
-_XONFIG_MAIN_ACTIONS = {
-    "info": _info,
-    "wizard": _wizard,
-    "styles": _styles,
-    "colors": _colors,
-    "tutorial": _tutorial,
-}
+def _jupyter_kernel(
+    user=False,
+    prefix=None,
+    root=None,
+):
+    """Generate xonsh kernel for jupyter.
+
+    Parameters
+    ----------
+    user : -u, --user
+        Install kernel spec in user config directory.
+    prefix : -p, --prefix
+        Installation prefix for bin, lib, etc.
+    root : -r, --root
+        Install relative to this alternate root directory.
+    """
+    try:
+        from jupyter_client.kernelspec import KernelSpecManager, NoSuchKernel
+    except ImportError as e:
+        raise ImportError("Jupyter not found in current Python environment") from e
+
+    ksm = KernelSpecManager()
+
+    prefix = prefix or sys.prefix
+    spec = {
+        "argv": [
+            sys.executable,
+            "-m",
+            "xonsh.jupyter_kernel",
+            "-f",
+            "{connection_file}",
+        ],
+        "display_name": "Xonsh",
+        "language": "xonsh",
+        "codemirror_mode": "shell",
+    }
+
+    if root and prefix:
+        # os.path.join isn't used since prefix is probably absolute
+        prefix = root + prefix
+
+    try:
+        old_jup_kernel = ksm.get_kernel_spec(XONSH_JUPYTER_KERNEL)
+        if not old_jup_kernel.resource_dir.startswith(prefix):
+            print(
+                "Removing existing Jupyter kernel found at {0}".format(
+                    old_jup_kernel.resource_dir
+                )
+            )
+        ksm.remove_kernel_spec(XONSH_JUPYTER_KERNEL)
+    except NoSuchKernel:
+        pass
+
+    if sys.platform == "win32":
+        # Ensure that conda-build detects the hard coded prefix
+        spec["argv"][0] = spec["argv"][0].replace(os.sep, os.altsep)
+        prefix = prefix.replace(os.sep, os.altsep)
+
+    with tempfile.TemporaryDirectory() as d:
+        os.chmod(d, 0o755)  # Starts off as 700, not user readable
+        with open(os.path.join(d, "kernel.json"), "w") as f:
+            json.dump(spec, f, sort_keys=True)
+
+        print("Installing Jupyter kernel spec:")
+        print("  root: {0!r}".format(root))
+        if user:
+            print("  as user: {0}".format(user))
+        elif root and prefix:
+            print("  combined prefix {0!r}".format(prefix))
+        else:
+            print("  prefix: {0!r}".format(prefix))
+        ksm.install_kernel_spec(
+            d, XONSH_JUPYTER_KERNEL, user=user, prefix=(None if user else prefix)
+        )
+        return 0
 
 
-def xonfig_main(args=None):
-    """Main xonfig entry point."""
-    if not args or (
-        args[0] not in _XONFIG_MAIN_ACTIONS and args[0] not in {"-h", "--help"}
-    ):
-        args.insert(0, "info")
-    parser = _xonfig_create_parser()
-    ns = parser.parse_args(args)
-    if ns.action is None:  # apply default action
-        ns = parser.parse_args(["info"] + args)
-    return _XONFIG_MAIN_ACTIONS[ns.action](ns)
+class XonfigAlias(ArgParserAlias):
+    """Manage xonsh configuration."""
+
+    def build(self):
+        parser = self.create_parser(prog="xonfig")
+        # register as default action
+        add_args(parser, _info, allowed_params=())
+        parser.add_command(_info)
+        parser.add_command(_web)
+        parser.add_command(_wizard)
+        parser.add_command(_styles)
+        parser.add_command(_colors)
+        parser.add_command(_tutorial)
+        parser.add_command(_jupyter_kernel)
+
+        return parser
+
+
+xonfig_main = XonfigAlias()
 
 
 @lazyobject
@@ -678,7 +811,7 @@ def STRIP_COLOR_RE():
 
 
 def _align_string(string, align="<", fill=" ", width=80):
-    """ Align and pad a color formatted string """
+    """Align and pad a color formatted string"""
     linelen = len(STRIP_COLOR_RE.sub("", string))
     padlen = max(width - linelen, 0)
     if align == "^":
@@ -705,7 +838,6 @@ def TAGLINES():
         "Piggy glanced nervously into hell and cradled the xonsh",
         "The xonsh is a symbol",
         "It is pronounced conch",
-        "The shell, bourne again",
         "Snailed it",
         "Starfish loves you",
         "Come snail away",
@@ -726,34 +858,50 @@ def TAGLINES():
         "Python-powered, cross-platform, Unix-gazing shell",
         "Tab completion in Alderaan places",
         "This fix was trickier than expected",
-        "The unholy cross of Bash/Python",
     ]
 
 
 # list of strings or tuples (string, align, fill)
 WELCOME_MSG = [
     "",
-    ("{{INTENSE_WHITE}}Welcome to the xonsh shell ({version}){{NO_COLOR}}", "^", " "),
+    ("Welcome to the xonsh shell {version}", "^", " "),
     "",
-    ("{{INTENSE_RED}}~{{NO_COLOR}} {tagline} {{INTENSE_RED}}~{{NO_COLOR}}", "^", " "),
+    ("{{INTENSE_RED}}~{{RESET}} {tagline} {{INTENSE_RED}}~{{RESET}}", "^", " "),
+    "",
+    ("{{INTENSE_BLACK}}", "<", "-"),
+    "",
+    (
+        "{{INTENSE_BLACK}}Create ~/.xonshrc file manually or use xonfig to suppress the welcome message",
+        "^",
+        " ",
+    ),
+    "",
+    "{{INTENSE_BLACK}}Start from commands:",
+    "  {{GREEN}}xonfig{{RESET}} web         {{INTENSE_BLACK}}# Run the configuration tool in the browser to create ~/.xonshrc {{RESET}}",
+    "  {{GREEN}}xonfig{{RESET}} tutorial    {{INTENSE_BLACK}}# Open the xonsh tutorial in the browser{{RESET}}",
+    "[SHELL_TYPE_WARNING]",
     "",
     ("{{INTENSE_BLACK}}", "<", "-"),
-    "{{GREEN}}xonfig{{NO_COLOR}} tutorial    {{INTENSE_WHITE}}->    Launch the tutorial in "
-    "the browser{{NO_COLOR}}",
-    "{{GREEN}}xonfig{{NO_COLOR}} wizard      {{INTENSE_WHITE}}->    Run the configuration "
-    "wizard and claim your shell {{NO_COLOR}}",
-    "{{INTENSE_BLACK}}(Note: Run the Wizard or create a {{RED}}~/.xonshrc{{INTENSE_BLACK}} file "
-    "to suppress the welcome screen)",
     "",
 ]
 
 
 def print_welcome_screen():
+    shell_type = XSH.env.get("SHELL_TYPE")
     subst = dict(tagline=random.choice(list(TAGLINES)), version=XONSH_VERSION)
     for elem in WELCOME_MSG:
+        if elem == "[SHELL_TYPE_WARNING]":
+            if shell_type != "prompt_toolkit":
+                print_color(
+                    f"\n{{INTENSE_BLACK}}You are currently using the {shell_type} backend. "
+                    f"For interactive tab-completion, on-the-fly syntax highlighting, and more, install prompt_toolkit by running:\n\n"
+                    f"  {{GREEN}}xpip{{RESET}} install -U 'xonsh[full]'"
+                )
+            continue
         if isinstance(elem, str):
             elem = (elem, "", "")
         line = elem[0].format(**subst)
         termwidth = os.get_terminal_size().columns
         line = _align_string(line, elem[1], elem[2], width=termwidth)
         print_color(line)
+    print_color("{RESET}", end="")
