@@ -2,9 +2,14 @@
 This modules is the place where one would define the xontribs.
 """
 
+import ast
 import functools
+import importlib.util
+from pathlib import Path
 import typing as tp
 
+from xonsh.lazyasd import LazyObject, lazyobject
+
 
 class _XontribPkg(tp.NamedTuple):
     """Class to define package information of a xontrib.
@@ -43,11 +48,20 @@ class Xontrib(tp.NamedTuple):
     """
 
     url: str = ""
-    description: str = ""
+    description: tp.Union[str, LazyObject] = ""
     package: tp.Optional[_XontribPkg] = None
     tags: tp.Tuple[str, ...] = ()
 
 
+def get_module_docstring(module: str) -> str:
+    """Find the module and return its docstring without actual import"""
+
+    spec = importlib.util.find_spec(module)
+    if spec and spec.has_location and spec.origin:
+        return ast.get_docstring(ast.parse(Path(spec.origin).read_text())) or ""
+    return ""
+
+
 @functools.lru_cache()
 def get_xontribs() -> tp.Dict[str, Xontrib]:
     """Return xontrib definitions lazily."""
@@ -70,20 +84,7 @@ def define_xontribs():
     return {
         "abbrevs": Xontrib(
             url="http://xon.sh",
-            description="Adds ``abbrevs`` dictionary to hold user-defined "
-            "command abbreviations. The dictionary is searched "
-            "as you type and the matching words are replaced "
-            "at the command line by the corresponding "
-            "dictionary contents once you hit 'Space' or "
-            "'Return' key. For instance a frequently used "
-            "command such as ``git status`` can be abbreviated "
-            "to ``gst`` as follows::\n"
-            "\n"
-            "    $ xontrib load abbrevs\n"
-            "    $ abbrevs['gst'] = 'git status'\n"
-            "    $ gst # Once you hit <space> or <return>, "
-            "'gst' gets expanded to 'git status'.\n"
-            "\n",
+            description=lazyobject(lambda: get_module_docstring("xontrib.abbrevs")),
             package=core_pkg,
         ),
         "apt_tabcomplete": Xontrib(
@@ -109,7 +110,7 @@ def define_xontribs():
             ),
         ),
         "autojump": Xontrib(
-            url="https://github.com/sagartewari01/autojump-xonsh",
+            url="https://github.com/wshanks/xontrib-autojump",
             description="autojump support for xonsh",
         ),
         "autovox": Xontrib(
@@ -140,7 +141,7 @@ def define_xontribs():
             ),
         ),
         "avox_poetry": Xontrib(
-            url="github.com/jnoortheen/xontrib-avox-poetry",
+            url="https://github.com/jnoortheen/xontrib-avox-poetry",
             description="auto-activate venv as one cd into a poetry project folder. "
             "Activate ``.venv`` inside the project folder is also supported.",
             package=_XontribPkg(
@@ -192,7 +193,7 @@ def define_xontribs():
             package=core_pkg,
         ),
         "broot": Xontrib(
-            url="github.com/jnoortheen/xontrib-broot",
+            url="https://github.com/jnoortheen/xontrib-broot",
             description="supports broot with br alias",
             package=_XontribPkg(
                 name="xontrib-broot",
@@ -202,13 +203,24 @@ def define_xontribs():
             ),
         ),
         "powerline3": Xontrib(
-            url="github.com/jnoortheen/xontrib-powerline3",
+            url="https://github.com/jnoortheen/xontrib-powerline3",
             description="Powerline theme with native $PROMPT_FIELDS support.",
             package=_XontribPkg(
                 name="xontrib-powerline3",
                 license="MIT",
                 install={"pip": "xpip install xontrib-powerline3"},
-                url="https://github.com/jnoortheen/xontrib-broot",
+                url="https://github.com/jnoortheen/xontrib-powerline3",
+            ),
+        ),
+        "cd": Xontrib(
+            url="https://github.com/eugenesvk/xontrib-cd",
+            description="'cd' to any path without escaping in xonsh shell "
+            "('cd '→'cd! ')",
+            package=_XontribPkg(
+                name="xontrib-cd",
+                license="MIT",
+                install={"pip": "xpip install xontrib-cd"},
+                url="https://github.com/eugenesvk/xontrib-cd",
             ),
         ),
         "cmd_done": Xontrib(
@@ -337,6 +349,17 @@ def define_xontribs():
                 url="https://github.com/dyuri/xontrib-gitinfo",
             ),
         ),
+        "history_encrypt": Xontrib(
+            url="https://github.com/anki-code/xontrib-history-encrypt",
+            description="History backend that encrypt the xonsh shell commands history "
+            "to prevent leaking sensitive data.",
+            package=_XontribPkg(
+                name="xontrib-history-encrypt",
+                license="MIT",
+                install={"pip": "xpip install xontrib-history-encrypt"},
+                url="https://github.com/anki-code/xontrib-history-encrypt",
+            ),
+        ),
         "hist_navigator": Xontrib(
             url="https://github.com/jnoortheen/xontrib-hist-navigator",
             description="Move through directory history with nextd "
@@ -360,6 +383,16 @@ def define_xontribs():
                 url="https://github.com/con-f-use/xontrib-histcpy",
             ),
         ),
+        "homebrew": Xontrib(
+            url="https://github.com/eugenesvk/xontrib-homebrew",
+            description="Add Homebrew's shell environment to xonsh shell on macOS/Linux",
+            package=_XontribPkg(
+                name="xontrib-homebrew",
+                license="MIT",
+                install={"pip": "xpip install xontrib-homebrew"},
+                url="https://github.com/eugenesvk/xontrib-homebrew",
+            ),
+        ),
         "jedi": Xontrib(
             url="http://xon.sh",
             description="Use Jedi as xonsh's python completer.",
@@ -378,6 +411,16 @@ def define_xontribs():
                 url="https://github.com/scopatz/xontrib-kitty",
             ),
         ),
+        "macro_lib": Xontrib(
+            url="https://github.com/anki-code/xontrib-macro-lib",
+            description="Library of the useful macros for the xonsh shell.",
+            package=_XontribPkg(
+                name="xontrib-macro-lib",
+                license="BSD",
+                install={"pip": "xpip install xontrib-macro-lib"},
+                url="https://github.com/anki-code/xontrib-macro-lib",
+            ),
+        ),
         "mpl": Xontrib(
             url="http://xon.sh",
             description="Matplotlib hooks for xonsh, including the new 'mpl' "
@@ -473,6 +516,16 @@ def define_xontribs():
             description="Adds return code info to the prompt",
             package=core_pkg,
         ),
+        "prompt_starship": Xontrib(
+            url="https://github.com/anki-code/xontrib-prompt-starship",
+            description="Starship prompt in xonsh shell.",
+            package=_XontribPkg(
+                name="xontrib-prompt-starship",
+                license="MIT",
+                install={"pip": "xpip install xontrib-prompt-starship"},
+                url="https://github.com/anki-code/xontrib-prompt-starship",
+            ),
+        ),
         "prompt_vi_mode": Xontrib(
             url="https://github.com/t184256/xontrib-prompt-vi-mode",
             description="vi-mode status formatter for xonsh prompt",
@@ -493,6 +546,16 @@ def define_xontribs():
                 url="https://github.com/dyuri/xontrib-pyenv",
             ),
         ),
+        "pyrtn": Xontrib(
+            url="https://github.com/dyuri/xontrib-pyrtn",
+            description="IPython like In[]/Out[] to access python return values in the current session.",
+            package=_XontribPkg(
+                name="xontrib-pyrtn",
+                license="MIT",
+                install={"pip": "xpip install xontrib-pyrtn"},
+                url="https://github.com/dyuri/xontrib-pyrtn",
+            ),
+        ),
         "readable-traceback": Xontrib(
             url="https://github.com/6syun9/xontrib-readable-traceback",
             description="Make traceback easier to see for " "xonsh.",
@@ -544,6 +607,16 @@ def define_xontribs():
                 url="https://github.com/dyuri/xontrib-ssh-agent",
             ),
         ),
+        "tcg": Xontrib(
+            url="https://github.com/zasdfgbnm/tcg/tree/master/shells/xonsh",
+            description="tcg integration.",
+            package=_XontribPkg(
+                name="xonsh-tcg",
+                license="MIT",
+                install={"pip": "xpip install xonsh-tcg"},
+                url="https://github.com/zasdfgbnm/tcg/tree/master/shells/xonsh",
+            ),
+        ),
         "vox": Xontrib(
             url="http://xon.sh",
             description="Python virtual environment manager for xonsh.",
