@@ -1,12 +1,13 @@
 """Implements the which xoreutil."""
 import os
 import argparse
-import builtins
 import functools
 
+import xonsh
 from xonsh.xoreutils import _which
 import xonsh.platform as xp
 import xonsh.procs.pipelines as xpp
+from xonsh.built_ins import XSH
 
 
 @functools.lru_cache()
@@ -75,7 +76,7 @@ def _which_create_parser():
 
 def print_global_object(arg, stdout):
     """Print the object."""
-    obj = builtins.__xonsh__.ctx.get(arg)
+    obj = XSH.ctx.get(arg)
     print("global object of {}".format(type(obj)), file=stdout)
 
 
@@ -85,9 +86,9 @@ def print_path(abs_name, from_where, stdout, verbose=False, captured=False):
         # Use list dir to get correct case for the filename
         # i.e. windows is case insensitive but case preserving
         p, f = os.path.split(abs_name)
-        f = next(s.name for s in xp.scandir(p) if s.name.lower() == f.lower())
+        f = next(s.name for s in os.scandir(p) if s.name.lower() == f.lower())
         abs_name = os.path.join(p, f)
-        if builtins.__xonsh__.env.get("FORCE_POSIX_PATHS", False):
+        if XSH.env.get("FORCE_POSIX_PATHS", False):
             abs_name.replace(os.sep, os.altsep)
     if verbose:
         print(f"{abs_name} ({from_where})", file=stdout)
@@ -98,19 +99,22 @@ def print_path(abs_name, from_where, stdout, verbose=False, captured=False):
 
 def print_alias(arg, stdout, verbose=False):
     """Print the alias."""
+    alias = XSH.aliases[arg]
     if not verbose:
-        if not callable(builtins.aliases[arg]):
-            print(" ".join(builtins.aliases[arg]), file=stdout)
+        if not callable(alias):
+            print(" ".join(alias), file=stdout)
+        elif isinstance(alias, xonsh.aliases.ExecAlias):
+            print(alias.src, file=stdout)
         else:
-            print(arg, file=stdout)
+            print(alias, file=stdout)
     else:
         print(
-            "aliases['{}'] = {}".format(arg, builtins.aliases[arg]),
+            "aliases['{}'] = {}".format(arg, alias),
             flush=True,
             file=stdout,
         )
-        if callable(builtins.aliases[arg]):
-            builtins.__xonsh__.superhelp(builtins.aliases[arg])
+        if callable(alias) and not isinstance(alias, xonsh.aliases.ExecAlias):
+            XSH.superhelp(alias)
 
 
 def which(args, stdin=None, stdout=None, stderr=None, spec=None):
@@ -137,16 +141,16 @@ def which(args, stdin=None, stdout=None, stderr=None, spec=None):
         if pargs.exts:
             exts = pargs.exts
         else:
-            exts = builtins.__xonsh__.env["PATHEXT"]
+            exts = XSH.env["PATHEXT"]
     else:
         exts = None
     failures = []
     for arg in pargs.args:
         nmatches = 0
-        if pargs.all and arg in builtins.__xonsh__.ctx:
+        if pargs.all and arg in XSH.ctx:
             print_global_object(arg, stdout)
             nmatches += 1
-        if arg in builtins.aliases and not pargs.skip:
+        if arg in XSH.aliases and not pargs.skip:
             print_alias(arg, stdout, verbose)
             nmatches += 1
             if not pargs.all:
@@ -155,7 +159,7 @@ def which(args, stdin=None, stdout=None, stderr=None, spec=None):
         # from os.environ so we temporarily override it with
         # __xosnh_env__['PATH']
         original_os_path = xp.os_environ["PATH"]
-        xp.os_environ["PATH"] = builtins.__xonsh__.env.detype()["PATH"]
+        xp.os_environ["PATH"] = XSH.env.detype()["PATH"]
         matches = _which.whichgen(arg, exts=exts, verbose=verbose)
         for abs_name, from_where in matches:
             print_path(abs_name, from_where, stdout, verbose, captured)
